/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDAPPUI_H__
#define BEMUSEDAPPUI_H__

#include <eikapp.h>
#include <eikdoc.h>
#include <e32std.h>
#include <coeccntx.h>

#ifdef BEMUSED_SERIES_60
  #include <aknviewappui.h>
  #include <akntabgrp.h>
  #include <aknnavide.h>
#else
  #include <qikappui.h>
#endif

#include "BluetoothHandler.h"
#include "IncomingCallListener.h"

class CBemusedContainer;
class CBemusedAppUi;

#ifndef BEMUSED_SERIES_60

  class CBemusedMainView;
  class CBemusedFlipClosedView;

  const TInt KFullScreenWidth = 208;
  const TInt KFullScreenHeight = 222;

  const TReal KSkinScalingFactor = 1.182;

#else

  class CBemusedBrowseView;
  class CBemusedControlView;
  class CBemusedDisplaySettingsView;
  class CBemusedBehaviourSettingsView;
  class CBemusedPlaylistView;

  const TInt KFullScreenWidth = 176;
  const TInt KFullScreenHeight = 188;

#endif

const TUint8 KVolumeStep = 20;
const TUint8 KStartVolume = 130;

const TUint KBemusedDrawBackground = 0x1;
const TUint KBemusedDrawTrackName = 0x2;
const TUint KBemusedDrawTrackTime = 0x4;
const TUint KBemusedDrawButtons = 0x8;
const TUint KBemusedDrawShuffle = 0x10;
const TUint KBemusedDrawRepeat = 0x20;
const TUint KBemusedDrawVolume = 0x40;
const TUint KBemusedDrawEverything = 0xFFFF;

_LIT(KBemusedName, "Bemused");

enum TSongState {
	ENoSongLoaded = 0,
	ESongPlaying,
	ESongPaused,
	ESongStopped
	};

class TBemusedSettings
	{
public:
	enum TActionOnIncomingCall
		{
		EDoNothing,
		EPause,
		EStop,
		EMute
		};
	TInt iActionOnIncomingCall;

	TInt iActionOnStopping;

	enum TRetrieveList
		{
		EAllAtOnce,
		EOneFolderAtATime
		};
	TInt iRetrieveList;

	enum TConnectToPc
		{
		EWhenRequired,
		EOnStartup,
		EAutoToSamePc
		};
	TInt iConnectToPc;

	enum TDisplayType
		{
		EInWindow,
		EFullScreen
		};
	TInt iDisplayType;

	enum TDisplayFiles
		{
		EDisplayAll,
		EDisplayMusic,
		EDisplayMp3s
		};
	TInt iDisplayFiles;

	enum TTimeDisplay
		{
		ECountsUp,
		ECountsDown
		};
	TInt iTimeDisplay;

	enum TSongBrowserFont
		{
		EHuge,
		ELarge,
		EMedium,
		ESmall
		};
	TInt iSongBrowserFont;

	TInt iSkinFileIndex;

	TInt iInstallSkinIndex;
	TInt iUninstallSkinIndex;

	enum TUseId3Tags
		{
		ENo,
		EYes
		};
	TInt iUseId3Tags;

	TInt iBluetoothChannel;
	TInt iDownloadDrive;

	enum TChangeView
		{
		EUsingJoystick,
		EUsingAbc,
		EJoystickAutoOnPlay,
		EAbcAutoOnPlay
		};
	TInt iChangeView;

	enum TSelectingItem
		{
		EPlaysIt,
		EAddsItToPlaylist,
		EPlaysThenAdds
		};
	TInt iSelectingItem;

	enum TConnectUsing
		{
		EBluetooth,
		EMrouter,
		EInternet
		};
	TInt iConnectUsing;

	TBuf<50> iServerAddress;

	TBTDevAddr iAutoConnectAddress;
	TInt iAutoConnectPort;
	};

#ifdef BEMUSED_SERIES_60
class CBemusedAppUi : public CAknViewAppUi,
#else
class CBemusedAppUi : public CQikAppUi,
#endif
	public MBluetoothListener, public MIncomingCallListener
    {
public: // Constructors and destructor
    void ConstructL();
    ~CBemusedAppUi();
    
public:
	virtual void RefreshFileListL(CDirTreeNode* aDirectory);
	virtual void ConnectComplete(TInt aError, const TBTDevAddr& aAddress, TInt aPort);
	virtual void CommandComplete(TBemusedCommand aCommand, TInt aError);
	virtual void SetInfo(TBool aPlaying, TBool aPaused, TInt aSongLength, TInt aCurrentTime,
		TBool aShuffle, TBool aRepeat, const TDesC8& aSongNameGuess);
	virtual void SetDetailedInfo(TInt aSampleRate, TInt aBitRate, TInt aChannels);
	virtual TBemusedSettings& Settings();
	virtual void SetPlaylistIndex(TInt aIndex);
	virtual void ReceivedVolume(TInt aVolume);
	virtual RPointerArray<HBufC8>& Playlist();
	virtual CCoeEnv* CoeEnv();

	void PlayFile(CDirTreeNode* aFileNode, const TDesC8& aFileName, TBool aAddToPlaylist);
	void DownloadFile(const TDesC8& aFileName);
	void SelectInPlaylist(TInt aIndex);

	void GetDirectoryList(CDirTreeNode* aDirectory);

	TInt LoadSettings();
	TInt SaveSettings();

	TInt ReloadSkin();
	void DrawControlView(TUint aDrawFlags);

	void SetSkinAuthor(const TDesC8& aAuthor);
	void SetSkinName(const TDesC& aFileName);

	CDirTreeNode* CurrentDirectory();
	void SetCurrentDirectory(CDirTreeNode* aDirectory);
	TUint DrawFlags();
	void SetDrawFlags(TUint aFlags);
	void SetDrawPending(TBool aPending);
	CBluetoothHandler* BluetoothHandler();

    void HandleCommandL(TInt aCommand);

#ifdef BEMUSED_SERIES_60
	CBemusedControlView* ControlView();
	CBemusedBrowseView* BrowseView();
	CBemusedPlaylistView* PlaylistView();
#else
	CBemusedMainView* ControlView();
	CBemusedMainView* BrowseView();
	CBemusedMainView* PlaylistView();

	void SetActiveView(TUid aViewUid);
#endif

	TSongState SongState();
	TInt& PlaylistIndex();
	TInt CurrentTime();
	TInt SongLength();
	TBool Shuffle();
	TBool Repeat();
	TInt CurrentVolume();
	void SetVolume(TInt aVolume);
	const TDesC& SongNameGuess();

	TFileName& SkinPathL();

	void GetPlaylist();

public: // from MIncomingCallListener
	void IncomingCall(const TDesC& aName);

private:
    virtual TKeyResponse HandleKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

	static TInt SongTimerCallback(TAny* aPtr);
	void HandleSongTimer();

	void BuildIniPath(TFileName& aIniFileName);
	void AddFilesToPlaylist(CDirTreeNode* aNode);

	void DisplayDetailedInfoDialogL(TInt aSampleRate, TInt aBitRate, TInt aNumChannels);

private: //Data

#ifdef BEMUSED_SERIES_60
    CAknNavigationControlContainer* iNaviPane;
    CAknTabGroup*                   iTabGroup;
    CAknNavigationDecorator*        iDecoratedTabGroup;
#endif

	CEikonEnv* iEikEnv;

	CPeriodic* iSongTimer;
	CIncomingCallListener* iIncomingCallListener;

#ifdef BEMUSED_SERIES_60
	CBemusedBrowseView* iBrowseView;
	CBemusedControlView* iControlView;
	CBemusedDisplaySettingsView* iDisplaySettingsView;
	CBemusedBehaviourSettingsView* iBehaviourSettingsView;
	CBemusedPlaylistView* iPlaylistView;
#else
	CBemusedMainView* iMainView;
	CBemusedFlipClosedView* iFlipClosedView;
	TUid iActiveView;
#endif

	CBluetoothHandler* iBluetoothHandler;
	CDirTreeNode* iCurrentDirectory;
	TUint8 iCurrentVolume;
	CDirTreeNode* iCurrentFile;
	TInt iSongLength;
	TInt iCurrentTime;

	TSongState iSongState;
	TBool iShuffle;
	TBool iRepeat;
	TInt iSampleRate;
	TInt iBitRate;
	TInt iChannels;
	
	TBemusedSettings iSettings;

	TUint iDrawFlags;
	TBool iDrawPending;

	RPointerArray<HBufC8> iPlaylist;
	TInt iPlaylistIndex;

	TFileName iSkinPath;
	TFileName iSongNameGuess;

	TBTDevAddr iLastDeviceAddress;
	TInt iLastPort;

	TBuf8<128> iSkinAuthor;
	TFileName iSkinName;
    };

#endif

// End of File
