/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDBROWSECONTAINER_H__
#define BEMUSEDBROWSECONTAINER_H__

#include <coecntrl.h>
#include <eikenv.h>
#include "BemusedListBox.h"

class CBemusedAppUi;
class CDirTreeNode;

enum TMusicFileType
	{
	EUnknownFile,
	EMusicFile,
	EMp3File
	};

class CBemusedBrowseContainer : public CCoeControl, public MCoeControlObserver
#ifndef BEMUSED_SERIES_60
, public MEikListBoxObserver
#endif
    {
public: // Constructors and destructor
    void ConstructL(CBemusedAppUi* aAppUi, const TRect& aRect);
    ~CBemusedBrowseContainer();

public: // New functions
	void FileSelectedL();
	void PlaySelectedFileL(TBool aAddToPlaylist);
	void DownloadSelectedFileL();
	void RefreshFileListL(CDirTreeNode* aDirectory);
	void UpOneLevelL();
	void GetFullNodeFileName(CDirTreeNode* aNode, TDes8& aFileName);
	void SetListEmptyStringL(const TDesC& aEmptyString);
	void ScrollSelectedSong();
	void FindSongL(TBool aFindNext);

	CDirTreeNode* RecursiveFind(CDirTreeNode* aNode, TDes8& aName);

	void SetColours(const TRgb& aTextForeground, const TRgb& aTextBackground,
		const TRgb& aHighlightForeground, const TRgb& aHighlightBackground);

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

#ifndef BEMUSED_SERIES_60
	void SelectNextFile();
	void SelectPreviousFile();
#endif

private: // Functions from base classes
    void SizeChanged();
    TInt CountComponentControls() const;
    CCoeControl* ComponentControl(TInt aIndex) const;
	void Draw(const TRect& aRect) const;
	void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);

#ifndef BEMUSED_SERIES_60
	virtual void HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType);
#endif

private: // New functions.
	CDirTreeNode* FindSelectedItem();
	TMusicFileType GetFileType(const TDesC& aFileName);
	TBool FileShouldBeDisplayed(const CDirTreeNode* aNode);
    
private: // Data.
	CBemusedListBox* iListBox;
	CEikScrollBarFrame* iScrollBarFrame;
	CDesC16ArrayFlat* iListItems;
	
	CBemusedAppUi* iAppUi;

	CListBoxData* iListBoxData;
	CFont* iFont;

	TRgb iTextForegroundColour;
	TRgb iTextBackgroundColour;
	TRgb iHighlightForegroundColour;
	TRgb iHighlightBackgroundColour;

	TInt iSongOffsetIndex;

	TFileName iLastSearchedSong;
	TInt iFindResultsToSkip;
	TInt iResultsFound;
    };

#endif

// End of File
