/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDCONTROLCONTAINER_H__
#define BEMUSEDCONTROLCONTAINER_H__

#include <coecntrl.h>
#include <eikcmbut.h>

#include "BemusedAppUi.h"

#ifdef BEMUSED_SERIES_60
class CBemusedControlView;
#else
class CBemusedMainView;
#endif

#ifdef POWERPOINT_ONLY
const TInt KNumButtons = 4;
#else
const TInt KNumButtons = 5;
#endif

class CBemusedControlContainer : public CCoeControl, MCoeControlObserver
    {
public: // Constructors and destructor
#ifdef BEMUSED_SERIES_60
	void ConstructL(CBemusedAppUi* aAppUi, CBemusedControlView* aView, const TRect& aRect);
#else
	void ConstructL(CBemusedAppUi* aAppUi, CBemusedMainView* aView, const TRect& aRect);
#endif
	~CBemusedControlContainer();

public: // Inherited from base classes.
	virtual TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

public: // New functions.
	void DrawTrackName(const TRect& aRect) const;
	void DrawTrackTime(const TRect& aRect) const;
	void DrawButtons(const TRect& aRect) const;
	void DrawShuffle(const TRect& aRect) const;
	void DrawRepeat(const TRect& aRect) const;
	void DrawVolume(const TRect& aRect) const;

	void AppInitiatedDraw() const;
	void DrawRequiredControls(TUint aRequired) const;

	TBool CheckFastForwardAndRewind();

private: // Functions from base classes
	void SizeChanged();
	TInt CountComponentControls() const;
	CCoeControl* ComponentControl(TInt aIndex) const;
	void Draw(const TRect& aRect) const;
	void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

#ifndef BEMUSED_SERIES_60
	virtual void HandlePointerEventL(const TPointerEvent& aPointerEvent);
#endif

private: // New functions.
	void ExecuteCommand(TInt aIndex);
	void GetVolumeSliderArea(TRect& aVolumeSliderArea) const;

#ifndef BEMUSED_SERIES_60
	TPoint AbsolutePoint(const TPoint& aPoint);
#endif

private: // Unowned data.
#ifdef BEMUSED_SERIES_60
	CBemusedControlView* iView;
#else
	CBemusedMainView* iView;
#endif

private: // Owned data.
	TInt iFocusIndex;
	TBool iButtonPressed;
	TBool iCheckedButtonPressedAlready;
	CBemusedAppUi* iAppUi;

	CFbsBitmap* iTrackNameBitmap;
	CFbsBitGc* iTrackNameContext;
	CFbsBitmapDevice* iTrackNameDevice;

	CFbsBitmap* iTrackTimeBitmap;
	CFbsBitGc* iTrackTimeContext;
	CFbsBitmapDevice* iTrackTimeDevice;

	TInt iWidth;
	TInt iHeight;
    };

#endif

// End of File
