/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDLISTBOX_H__
#define BEMUSEDLISTBOX_H__

#include <coecntrl.h>
#include <eikenv.h>
#include <eiktxlbx.h>
#include <eiklbi.h>

class CBemusedAppUi;
class CDirTreeNode;

const TInt KItemHeight = 20;

class CBemusedListBoxView : public CListBoxView
	{
public:
	CBemusedListBoxView();

	virtual void DrawEmptyList(const TRect &aClientRect) const;
	virtual void Draw(const TRect* aClipRect) const;
	};

// The class which is responsible for drawing the items in the list of songs. It needs to exist
// because CTextListItemDrawer doesn't give us enough control over the appearance of the text that's
// drawn.
class CBemusedListItemDrawer : public CTextListItemDrawer
	{
public:
	CBemusedListItemDrawer(MTextListBoxModel* aTextListBoxModel, const CFont* aFont);

	virtual void DrawActualItem(TInt aItemIndex, const TRect& aActualItemRect, TBool aItemIsCurrent,
		TBool aViewIsEmphasized, TBool, TBool aItemIsSelected) const;

	void SetColours(const TRgb& aTextForeground, const TRgb& aTextBackground,
		const TRgb& aHighlightForeground, const TRgb& aHighlightBackground);

	TBool ScrollSong(TInt aItemIndex);
	void ResetScrolling();

private:
	TRgb iTextForegroundColour;
	TRgb iTextBackgroundColour;
	TRgb iHighlightForegroundColour;
	TRgb iHighlightBackgroundColour;

	const CFont* iPersistentFont;

	TInt iScrollOffset;
	};

// A list box class whose only responsibility is to look after a CBemusedListItemDrawer.
class CBemusedListBox : public CEikTextListBox
	{
public:
	CBemusedListBox(const CFont* aFont);

protected:
	virtual void CreateItemDrawerL();
	virtual CListBoxView* MakeViewClassInstanceL();

private:
	const CFont* iFont;
	};

#endif

// End of File
