/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDSKINHANDLER_H__
#define BEMUSEDSKINHANDLER_H__

#include <MdaImageConverter.h>

const TInt KMaxSkinBitmapWidth = 176;
const TInt KMaxSkinBitmapHeight = 300;

enum TBemusedFont
	{
	EBemusedNoFont = 0,
	EBemusedTitleFont = 1,
	EBemusedDenseFont = 2,
	EBemusedLegendFont = 3,
	EBemusedAnnotationFont = 4,
	EBemusedSymbolFont = 5,
	EBemusedNormalFont = 6
	};

class TBemusedSkin
	{
public:
	TRect iButtonsRect;
	TRect iTrackTimeRect;
	TRect iTrackNameRect;
	TRect iVolumeRect;
	TSize iVolumeSliderSize;
	TRect iShuffleRect;
	TRect iRepeatRect;

	TRgb iTrackNameColour;
	TRgb iTrackTimeColour;
	TBemusedFont iTrackNameFont;
	TBemusedFont iTrackTimeFont;

	CFbsBitmap* iBitmap;

	TBool iValid;
	};

class CBemusedAppUi;

class CBemusedSkinHandler : public CBase, public MMdaImageUtilObserver
    {
public: // Constructors and destructor
	CBemusedSkinHandler(CBemusedAppUi* aAppUi, TBemusedSkin& aSkin);
	static CBemusedSkinHandler* NewL(CBemusedAppUi* aAppUi, TBemusedSkin& aSkin);

    ~CBemusedSkinHandler();

public: // From MMdaImageUtilObserver.
	virtual void MiuoConvertComplete(TInt aError);
	virtual void MiuoCreateComplete(TInt aError);
	virtual void MiuoOpenComplete(TInt aError);

public: // New functions.
	void LoadSkinL();
	static const CFont* BemusedFontToCFont(TBemusedFont aBemusedFont);

protected:
	void ConstructL();

private:
	void ReadSkinDescriptionL(const TDesC& aFileName);
	void ReadSkinColour(TLex8& aLex, TRgb& aColour);
	void ReadSkinRect(TLex8& aLex, TRect& aRect);

private: // Data
	CMdaImageFileToBitmapUtility* iSkinLoader;
	CMdaBitmapScaler* iSkinScaler;
	
	CBemusedAppUi* iAppUi;
	TBemusedSkin& iSkin;

	TBool iScaling;
    };

#endif

// End of File
