/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSED_DISPLAY_SETTINGS_CONTAINER_H__
#define BEMUSED_DISPLAY_SETTINGS_CONTAINER_H__

#include <coecntrl.h>
#include <eikenv.h>
#include <aknlists.h>
#include <aknsettingitemlist.h>

#include "BemusedAppUi.h"

class CBemusedDisplaySettingsView;

// The location of the Inbox.
_LIT(KInboxPath, "\\system\\mail\\00001001_S");

// A wildcard which matches all files in a directory.
_LIT(KMatchAll, "*");

// A wildcard which matches only directories containing files.
_LIT(KMatchFiles, "*_F");

_LIT(KMatchInWindowSkinBitmaps, "*_iw.png");
_LIT(KMatchFullScreenSkinBitmaps, "*_fs.png");
_LIT(KMatchInWindowSkins, "*_iw.*");
_LIT(KMatchFullScreenSkins, "*_fs.*");

const TInt KSkinSettingIndex = 1;
const TInt KInstallSkinSettingIndex = 5;
const TInt KUninstallSkinSettingIndex = 6;


class CBemusedDisplaySettingItemList : public CAknSettingItemList
	{
public:
	CBemusedDisplaySettingItemList(TBemusedSettings& aData);

protected:
	virtual CAknSettingItem* CreateSettingItemL(TInt aSettingId);

private:
	TBemusedSettings& iData;
	};


class CBemusedDisplaySettingsContainer : public CCoeControl, MCoeControlObserver
    {
public: // Constructors and destructor
    void ConstructL(CBemusedAppUi* aAppUi, CBemusedDisplaySettingsView* aView, const TRect& aRect);
    ~CBemusedDisplaySettingsContainer();

public: // Functions from base classes
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

	CBemusedDisplaySettingItemList* iListBox;

private: // Functions from base classes
    void SizeChanged();
    TInt CountComponentControls() const;
    CCoeControl* ComponentControl(TInt aIndex) const;
    void Draw(const TRect& aRect) const;
    void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

private: // New functions.
	void GetSkinListL(CAknEnumeratedTextPopupSettingItem* aSettingItem, TBool aAddSelectItem);
	void GetInboxSkinListL(CAknEnumeratedTextPopupSettingItem* aSettingItem);

	void InstallSkin(const TDesC& aBitmapFileName, const TDesC& aDescriptionFileName);
    
private: // Data.
	CDesC16ArrayFlat* iInboxSkinBitmapFileNames;
	CDesC16ArrayFlat* iInboxSkinDescriptionFileNames;

	CEikScrollBarFrame* iScrollBarFrame;
	
	CBemusedAppUi* iAppUi;
	CBemusedDisplaySettingsView* iView;
    };

#endif
