/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDPLAYLISTVIEW_H__
#define BEMUSEDPLAYLISTVIEW_H__

#include <aknview.h>
#include "BemusedListBox.h"

// UID of view
const TUid KPlaylistViewId = {3};

class CBemusedPlaylistView : public CAknView
    {
public: // Constructors and destructor
    void ConstructL();
    ~CBemusedPlaylistView();

public: // Functions from base classes
    TUid Id() const;
    void HandleCommandL(TInt aCommand);
    void HandleClientRectChange();

public: // New functions.
	void ReadSkinDescriptionL(const TDesC& aFileName);
	TInt GetSelectedFile();
	TBool IsFocused();
	void RefreshPlaylistL(const RPointerArray<HBufC8>& aPlaylist, TInt aIndex);
	void ScrollSelectedSong();
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
	void FindSongL(TBool aFindNext);

private: // From CAknView.
    void DoActivateL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,
        const TDesC8& aCustomMessage);
    void DoDeactivate();

private: // Defined here.
	void ReadSkinColour(TLex8& aLex, TRgb& aColour);

private: // Data.
    CBemusedListBox* iListBox;
	CEikScrollBarFrame* iScrollBarFrame;
	CDesC16ArrayFlat* iListItems;
	
	TRgb iTextForegroundColour;
	TRgb iTextBackgroundColour;
	TRgb iHighlightForegroundColour;
	TRgb iHighlightBackgroundColour;

	TBool iFocused;

	TFileName iLastSearchedSong;
	TInt iResultsFound;
	TInt iFindResultsToSkip;
    };

#endif

// End of File
