/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDFLIPCLOSEDVIEW_H__
#define BEMUSEDFLIPCLOSEDVIEW_H__

#include <coeview.h>
#include <coecntrl.h>
#include <eiksbfrm.h>

// UID of view
const TUid KFlipClosedViewId = {2};

class CBemusedBrowseContainer;
class CBemusedControlContainer;
class CBemusedSkinHandler;
class CBemusedAppUi;
class CQikTabScreen;
class CQikTabScreenPage;
class CEikLabel;
class TBemusedSkin;

class CBemusedFlipClosedView : public CCoeControl, public MCoeView, public MCoeControlObserver
    {
public: // Constructors and destructor
	CBemusedFlipClosedView(CBemusedAppUi* aAppUi);
    void ConstructL();
    ~CBemusedFlipClosedView();

public: // Functions from base classes
    TUid Id() const;
    void HandleCommandL(TInt aCommand);
    void HandleClientRectChange();
	virtual TInt CountComponentControls() const;
	virtual CCoeControl* ComponentControl(TInt aIndex) const;
	void Draw(const TRect& /*aRect*/) const;
	void AppInitiatedDraw() const;
	virtual TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);

private: // From MCoeView
	virtual TVwsViewId ViewId() const;
	virtual void ViewActivatedL(const TVwsViewId& aPrevViewId,
		TUid aCustomMessageId, const TDesC8& aCustomMessage);
	virtual void ViewDeactivated();
	virtual TVwsViewIdAndMessage ViewScreenDeviceChangedL();
	virtual TBool ViewScreenModeCompatible(TInt aScreenMode);

private: // From MCoeControlObserver
	virtual void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);

private:
	void DrawRequiredControls(TUint aRequired) const;
	void DrawTrackName(const TRect& aRect) const;
	void DrawTrackTime(const TRect& aRect) const;
	void DrawButtons(const TRect& aRect) const;

	void ExecuteCommand(TInt aIndex);

private: // Data.
	CBemusedAppUi* iAppUi;

	TInt iActivateCount;

	CBemusedSkinHandler* iSkinHandler;
	TBemusedSkin& iSkin;

	TInt iFocusIndex;
	TBool iButtonPressed;
	TBool iCheckedButtonPressedAlready;
    };

#endif

// End of File
