/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BEMUSEDMAINVIEW_H__
#define BEMUSEDMAINVIEW_H__

#include <coeview.h>
#include <coecntrl.h>
#include <eiksbfrm.h>

#include "BemusedSkinHandler.h"
#include "BemusedListBox.h"

// UID of view
const TUid KMainViewId = {1};

class CBemusedBrowseContainer;
class CBemusedControlContainer;
class CBemusedSkinHandler;
class CBemusedAppUi;
class CQikTabScreen;
class CQikTabScreenPage;

enum
	{
	EScreenModeFlipOpen = 0,
	EScreenModeFlipClosed
	};


class CBemusedMainView : public CCoeControl, public MCoeView, public MCoeControlObserver,
	public MEikListBoxObserver
    {
public: // Constructors and destructor
    void ConstructL(CBemusedAppUi* aAppUi);
    ~CBemusedMainView();

public: // Functions from base classes
    TUid Id() const;
    void HandleCommandL(TInt aCommand);
    void HandleClientRectChange();
	virtual TInt CountComponentControls() const;
	virtual CCoeControl* ComponentControl(TInt aIndex) const;

public: // New functions.
	void ReadSkinDescriptionL(const TDesC& aFileName);

	CBemusedBrowseContainer* BrowseContainer();
	CBemusedControlContainer* ControlContainer();
	void SetListLoaded(TBool aListLoaded);

	TBemusedSkin& Skin();
	CBemusedSkinHandler* SkinHandler();
	TInt TabIndex();
	CQikTabScreen* TabScreen();

	void RefreshPlaylistL(const RPointerArray<HBufC8>& aPlaylist, TInt aIndex);
	void FindSongInPlaylistL(TBool aFindNext);
	void SelectPreviousFileInPlaylist();
	void SelectNextFileInPlaylist();
	void PlaylistFileSelected();

private: // From MCoeView
	virtual TVwsViewId ViewId() const;
	virtual void ViewActivatedL(const TVwsViewId& aPrevViewId,
		TUid aCustomMessageId, const TDesC8& aCustomMessage);
	virtual void ViewDeactivated();
	virtual TVwsViewIdAndMessage ViewScreenDeviceChangedL();
	virtual TBool ViewScreenModeCompatible(TInt aScreenMode);

private: // From MCoeControlObserver
	virtual void HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);

private: // From MEikListBoxObserver
	virtual void HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType);

private: // Defined here.
	void ReadSkinColour(TLex8& aLex, TRgb& aColour);
	void CreatePlaylistBoxL();

private: // Data.
    CBemusedBrowseContainer* iBrowseContainer;
	CBemusedControlContainer* iControlContainer;

	CBemusedSkinHandler* iSkinHandler;
	TBemusedSkin iSkin;
	
	TRgb iTextForegroundColour;
	TRgb iTextBackgroundColour;
	TRgb iHighlightForegroundColour;
	TRgb iHighlightBackgroundColour;

	TBool iListLoaded;

	CBemusedAppUi* iAppUi;

	CQikTabScreen* iTabScreen;
	CQikTabScreenPage* iBrowsePage;
	CQikTabScreenPage* iControlPage;
	CQikTabScreenPage* iPlaylistPage;

	CBemusedListBox* iPlaylistBox;
	CEikScrollBarFrame* iPlaylistScrollBarFrame;
	CDesC16ArrayFlat* iPlaylistItems;
	
	TFileName iLastSearchedSong;
	TInt iResultsFound;
	TInt iFindResultsToSkip;

	TInt iActivateCount;
    };

#endif

// End of File
