/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include "BemusedApp.h"
#include "BemusedAppUi.h"
#include "BemusedBrowseContainer.h"
#include "BemusedControlContainer.h"
#include "bemused.hrh"

#ifdef BEMUSED_SERIES_60

  #include <avkon.hrh>
  #include <aknpopup.h>

  #include "BemusedBrowseView.h"
  #include "BemusedControlView.h"
  #include "BemusedPlaylistView.h"
  #include "BemusedDisplaySettingsView.h"
  #include "BemusedDisplaySettingsContainer.h"
  #include "BemusedBehaviourSettingsView.h"
  #include "BemusedBehaviourSettingsContainer.h"

  #include <Bemused.rsg>

#else

  #include <qiktabscreen.h>

  #include "BemusedMainView.h"
  #include "BemusedFlipClosedView.h"
  #include "BemusedSettingsDialog.h"
  #include "BemusedInstallSkinDialog.h"
  #include <Bemused.rsg>

#endif

CBemusedAppUi::~CBemusedAppUi()
	{
	iPlaylist.ResetAndDestroy();
	iPlaylist.Close();

	delete iIncomingCallListener;
	delete iSongTimer;
	delete iBluetoothHandler;

#ifdef BEMUSED_SERIES_60
    delete iDecoratedTabGroup;
#else
	delete iMainView;
	delete iFlipClosedView;
#endif
	}

TInt CBemusedAppUi::LoadSettings()
	{
	RFile file;
	TInt ret;

	TFileName iniFilename;
	BuildIniPath(iniFilename);

	ret = file.Open(iEikEnv->FsSession(), iniFilename, EFileRead | EFileShareAny);
	if (ret != KErrNone)
		return ret;

	// REMEMBER to change this number when you add new settings!
	TBuf8<114> settingsBuf;

	ret = file.Read(settingsBuf);
	file.Close();

	// Settings are simply implemented as a list of numbers in a text file.

	TLex8 lex(settingsBuf);
	lex.Val(iSettings.iActionOnIncomingCall);
	lex.SkipSpace();
	lex.Val(iSettings.iActionOnStopping);
	lex.SkipSpace();
	lex.Val(iSettings.iConnectToPc);
	lex.SkipSpace();
	lex.Val(iSettings.iDisplayType);
	lex.SkipSpace();
	lex.Val(iSettings.iTimeDisplay);
	lex.SkipSpace();
	lex.Val(iSettings.iSkinFileIndex);
	lex.SkipSpace();
	lex.Val(iSettings.iRetrieveList);
	lex.SkipSpace();
	lex.Val(iSettings.iSongBrowserFont);
	lex.SkipSpace();
	lex.Val(iSettings.iDisplayFiles);
	lex.SkipSpace();

	TInt volume = KStartVolume;
	lex.Val(volume);
	lex.SkipSpace();
	iCurrentVolume = (TUint8) volume;

	for (TInt i = 0; i < 6; i++)
		{
		TInt tempByte;
		lex.Val(tempByte);
		iSettings.iAutoConnectAddress[i] = (TUint8) tempByte;
		lex.SkipSpace();
		}

	lex.Val(iSettings.iAutoConnectPort);
	lex.SkipSpace();
	lex.Val(iSettings.iUseId3Tags);
	lex.SkipSpace();
	lex.Val(iSettings.iBluetoothChannel);
	lex.SkipSpace();
	lex.Val(iSettings.iDownloadDrive);
	lex.SkipSpace();
	lex.Val(iSettings.iChangeView);
	lex.SkipSpace();
	lex.Val(iSettings.iSelectingItem);
	lex.SkipSpace();
	lex.Val(iSettings.iConnectUsing);
	lex.SkipSpace();
	if (lex.Eos())
		iSettings.iServerAddress = KNullDesC;
	else
		iSettings.iServerAddress.Copy(lex.NextToken());

	return ret;
	}

TInt CBemusedAppUi::SaveSettings()
	{
	RFile file;
	TInt ret = KErrNone;

	TFileName iniFilename;
	BuildIniPath(iniFilename);

#ifdef BEMUSED_SERIES_60
	if (iDisplaySettingsView->SettingsContainer())
		TRAP(ret, iDisplaySettingsView->SettingsContainer()->iListBox->StoreSettingsL());
	if (iBehaviourSettingsView->SettingsContainer())
		TRAP(ret, iBehaviourSettingsView->SettingsContainer()->iListBox->StoreSettingsL());
#endif

	if (ret != KErrNone)
		return ret;

	ret = file.Replace(iEikEnv->FsSession(), iniFilename, EFileShareAny);
	if (ret != KErrNone)
		return ret;

	TBuf8<256> settingsBuf;
	settingsBuf.Format(_L8("%1d %1d %1d %1d %1d %02d %1d %1d %1d %03d"),
		iSettings.iActionOnIncomingCall,
		iSettings.iActionOnStopping, iSettings.iConnectToPc, iSettings.iDisplayType,
		iSettings.iTimeDisplay, iSettings.iSkinFileIndex, iSettings.iRetrieveList,
		iSettings.iSongBrowserFont, iSettings.iDisplayFiles, iCurrentVolume);

	ret = file.Write(settingsBuf);

	for (TInt i = 0; i < 6; i++)
		{
		settingsBuf.Format(_L8(" %03d"), iSettings.iAutoConnectAddress[i]);
		file.Write(settingsBuf);
		}

	TBuf8<50> serverAddress;
	serverAddress.Copy(iSettings.iServerAddress);

	settingsBuf.Format(_L8(" %02d %1d %02d %02d %1d %1d %1d %50S"), iSettings.iAutoConnectPort,
		iSettings.iUseId3Tags, iSettings.iBluetoothChannel, iSettings.iDownloadDrive,
		iSettings.iChangeView, iSettings.iSelectingItem, iSettings.iConnectUsing,
		&serverAddress);
	file.Write(settingsBuf);

	file.Close();

	return ret;
	}

void CBemusedAppUi::HandleCommandL(TInt aCommand)
    {
	CBemusedBrowseContainer* browseContainer = BrowseView()->BrowseContainer();

    switch (aCommand)
        {
        case EEikCmdExit:
			SaveSettings();
            Exit();
            break;

        case EBemusedCmdRefreshList:
			if (iSettings.iRetrieveList == TBemusedSettings::EAllAtOnce)
				iBluetoothHandler->RequestList(ETrue);
			else
				iBluetoothHandler->RequestList(EFalse);
            break;

		case EBemusedCmdPlay:
			if (browseContainer)
				browseContainer->PlaySelectedFileL(EFalse);
			break;

		case EBemusedCmdAddToPlaylist:
			if (browseContainer)
				browseContainer->PlaySelectedFileL(ETrue);
			break;

		case EBemusedCmdDownload:
			if (browseContainer)
				browseContainer->DownloadSelectedFileL();
			break;

		case EBemusedCmdShuffle:
			iBluetoothHandler->RequestShuffle(!iShuffle);
			break;

		case EBemusedCmdRepeat:
			iBluetoothHandler->RequestRepeat(!iRepeat);
			break;

		case EBemusedCmdFullScreen:
			iBluetoothHandler->RequestFullScreen();
			break;

		case EBemusedCmdShutDown:
			{
#ifndef POWERPOINT_ONLY
			TBuf<256> resourceBuf;
			iEikEnv->ReadResource(resourceBuf, R_SHUTDOWN_QUERY);
			if (iEikEnv->QueryWinL(resourceBuf, KNullDesC))
				iBluetoothHandler->RequestShutDown();
#endif
			}
			break;

		case EBemusedCmdFind:
			if (browseContainer)
				browseContainer->FindSongL(EFalse);
			break;

		case EBemusedCmdFindServer:
			iBluetoothHandler->RequestFindServer();
			break;


#ifdef BEMUSED_SERIES_60
		case EBemusedCmdExitDisplaySettings:
			ReloadSkin();
			// fall through!

		case EBemusedCmdExitBehaviourSettings:
			iTabGroup->SetActiveTabByIndex(1);
			ActivateLocalViewL(KControlViewId);
#endif
			break;

		case EBemusedCmdFindInPlaylist:
#ifdef BEMUSED_SERIES_60
			iPlaylistView->FindSongL(EFalse);
#else
			iMainView->FindSongInPlaylistL(EFalse);
#endif
			break;

		case EBemusedCmdTrackInfo:
			iBluetoothHandler->RequestDetailedTrackInfo();
			break;

		case EBemusedCmdPlayFromPlaylist:
#ifdef BEMUSED_SERIES_60
			if (iPlaylistView->IsFocused())
				{
				TInt selection = iPlaylistView->GetSelectedFile();
				if (selection != KErrNotFound)
					iBluetoothHandler->RequestSelectInPlaylist(selection);
				}
#endif
			break;

		case EBemusedCmdRefreshPlaylist:
			GetPlaylist();
			break;

		case EBemusedCmdRemoveAllFromPlaylist:
			iBluetoothHandler->RequestRemoveAllFromPlaylist();
			break;

		case EBemusedCmdShowSettings:
			{
#ifndef BEMUSED_SERIES_60
			CBemusedSettingsDialog* settingsDialog =
				new (ELeave) CBemusedSettingsDialog(this, iSettings);
			settingsDialog->ExecuteLD(R_BEMUSED_SETTINGS_DIALOG);
#endif
			}
			break;

#ifdef BEMUSED_SERIES_60
		case EBemusedCmdShowDisplaySettings:
			ActivateLocalViewL(KDisplaySettingsViewId);
			break;

		case EBemusedCmdShowBehaviourSettings:
			ActivateLocalViewL(KBehaviourSettingsViewId);
			break;
#endif

#ifndef BEMUSED_SERIES_60
		case EBemusedCmdInstallSkin:
			{
			CBemusedInstallSkinDialog* installSkinDialog =
				new (ELeave) CBemusedInstallSkinDialog(this, iSettings, ETrue);
			installSkinDialog->ExecuteLD(R_BEMUSED_INSTALL_SKIN_DIALOG);
			}
			break;

		case EBemusedCmdUninstallSkin:
			{
			CBemusedInstallSkinDialog* installSkinDialog =
				new (ELeave) CBemusedInstallSkinDialog(this, iSettings, EFalse);
			installSkinDialog->ExecuteLD(R_BEMUSED_INSTALL_SKIN_DIALOG);
			}
			break;
#endif

        default:
            break;      
        }
    }

void CBemusedAppUi::ConnectComplete(TInt aError, const TBTDevAddr& aAddress, TInt aPort)
	{
	if ((aError != KErrNone) && (aError != KErrCancel))
		{
		TBuf<256> formatBuf;
		TBuf<256> resourceBuf;
		iEikEnv->ReadResource(resourceBuf, R_CONNECT_FAILURE);

		formatBuf.Format(resourceBuf, aError);
		iEikEnv->InfoWinL(formatBuf, KNullDesC);
		}

	if (aError == KErrNone)
		{
		iSettings.iAutoConnectAddress = aAddress;
		iSettings.iAutoConnectPort = aPort;
		}

	DrawControlView(KBemusedDrawEverything);
	}

void CBemusedAppUi::CommandComplete(TBemusedCommand aCommand, TInt aError)
	{
	TInt err;

	if (aError != KErrNone)
	    {
		TBuf<256> formatBuf;

		switch (aError)
			{
		// Error codes which don't need any notification.
		case KErrCancel:
		case KErrTooBig:
			return;

		case KErrDisconnected:
		case KErrNotReady:
			iEikEnv->ReadResource(formatBuf, R_DISCONNECTED_ERROR);
			break;

		case KErrCouldNotConnect:
		case KErrHCIConnectFailed:
			iEikEnv->ReadResource(formatBuf, R_COULDNOTCONNECT_ERROR);
			break;

		case KErrCorrupt:
			iEikEnv->ReadResource(formatBuf, R_CORRUPT_ERROR);
			break;

		case KErrRfcommBadAddress:
		case -6004: // KHCIIoctlErrorBase - KHCIEncryptIoctl
			iEikEnv->ReadResource(formatBuf, R_BLUETOOTH_ERROR);
			break;

		case KErrTimedOut:
			iEikEnv->ReadResource(formatBuf, R_TIMEOUT_ERROR);
			break;

		case KErrPathNotFound:
			iEikEnv->ReadResource(formatBuf, R_PATHNOTFOUND_ERROR);
			break;

		default:
			{
			TBuf<256> resourceBuf;
			iEikEnv->ReadResource(resourceBuf, R_UNKNOWN_ERROR);
			formatBuf.Format(resourceBuf, aError);
			}
			break;
			}

        TRAP(err, iEikEnv->InfoWinL(KNullDesC, formatBuf));
		return;
		}

	if (aCommand == EBemusedCurrentTrackInfo)
		{
		DrawControlView(KBemusedDrawEverything & ~KBemusedDrawBackground);
		}
	else if (aCommand == EBemusedRemoveAllFromPlaylist)
		{
		iPlaylist.ResetAndDestroy();
		iPlaylistIndex = 0;
		}
	else if (aCommand == EBemusedGetPlaylist)
		{
#ifdef BEMUSED_SERIES_60
		if (iPlaylistView->IsFocused())
			TRAP(err, iPlaylistView->RefreshPlaylistL(iPlaylist, iPlaylistIndex));
#else
		TRAP(err, iMainView->RefreshPlaylistL(iPlaylist, iPlaylistIndex));
#endif
		}
	}

void CBemusedAppUi::RefreshFileListL(CDirTreeNode* aDirectory)
	{
	iCurrentDirectory = aDirectory;

#ifdef BEMUSED_SERIES_60
	iBrowseView->SetListLoaded(ETrue);
#endif

	if (BrowseView()->BrowseContainer())
		{
		BrowseView()->BrowseContainer()->RefreshFileListL(aDirectory);
		BrowseView()->BrowseContainer()->DrawNow();
		}

	// Start the timer. We have to do this here (and not any earlier) to make sure that any
	// asynchronous Bluetooth requests which happen on startup are complete.
	if (!iSongTimer->IsActive())
		iSongTimer->Start(0, 1000000, TCallBack(SongTimerCallback, this));
	}

void CBemusedAppUi::GetDirectoryList(CDirTreeNode* aDirectory)
	{
	iBluetoothHandler->RequestDirectoryList(aDirectory);
	}

void CBemusedAppUi::SetInfo(TBool aPlaying, TBool aPaused, TInt aSongLength, TInt aCurrentTime,
							TBool aShuffle, TBool aRepeat, const TDesC8& aSongNameGuess)
	{
	TSongState newSongState = ESongStopped;

	if (aPaused)
		newSongState = ESongPaused;
	else if (aPlaying)
		newSongState = ESongPlaying;

	if (newSongState != iSongState)
		{
//		iDrawFlags |= KBemusedDrawTrackName | KBemusedDrawTrackTime;
		iSongState = newSongState;
		DrawControlView(KBemusedDrawTrackName | KBemusedDrawTrackTime);
		}

	iCurrentTime = aCurrentTime;
	iSongLength = aSongLength;

	if (iShuffle != aShuffle)
		DrawControlView(KBemusedDrawShuffle);
	iShuffle = aShuffle;
	
	if (iRepeat != aRepeat)
		DrawControlView(KBemusedDrawRepeat);
	iRepeat = aRepeat;

	if (aSongNameGuess.Length() > iSongNameGuess.MaxLength())
		iSongNameGuess.Copy(aSongNameGuess.Left(iSongNameGuess.MaxLength()));
	else
		iSongNameGuess.Copy(aSongNameGuess);

#ifdef BEMUSED_SERIES_60
	if (iBehaviourSettingsView->SettingsContainer())
		iBehaviourSettingsView->SettingsContainer()->DrawDeferred();
	if (iDisplaySettingsView->SettingsContainer())
		iDisplaySettingsView->SettingsContainer()->DrawDeferred();
#endif
	}

void CBemusedAppUi::SetDetailedInfo(TInt aSampleRate, TInt aBitRate, TInt aChannels)
	{
	iSampleRate = aSampleRate;
	iBitRate = aBitRate;
	iChannels = aChannels;

	TRAPD(err, DisplayDetailedInfoDialogL(aSampleRate, aBitRate, aChannels));
	}

void CBemusedAppUi::PlayFile(CDirTreeNode* aFileNode, const TDesC8& aFileName,
							 TBool aAddToPlaylist)
	{
	CDirTreeNode* node = aFileNode->iFirstChild;
	HBufC8* fileName;

	// If the node we want to play is a directory, add each of its children to the
	// playlist.
	if (node)
		{
		if (!aAddToPlaylist)
			{
			iPlaylist.ResetAndDestroy();
			iPlaylistIndex = 0;
			}
		AddFilesToPlaylist(node);
		}
	else
		{
		fileName = aFileNode->iFileName->Alloc();
		if (fileName)
			{
			if (aAddToPlaylist)
				{
				iPlaylist.Append(fileName);
				}
			else
				{
				iPlaylist.ResetAndDestroy();
				iPlaylist.Append(fileName);
				iPlaylistIndex = 0;
				}
			}
		}

	if (aAddToPlaylist)
		iBluetoothHandler->RequestAddFileToPlaylist(aFileName);
	else
		iBluetoothHandler->RequestPlayFile(aFileName);

	// Move to the Control view if required.

	if (!aAddToPlaylist && ((iSettings.iChangeView == TBemusedSettings::EJoystickAutoOnPlay) ||
		(iSettings.iChangeView == TBemusedSettings::EAbcAutoOnPlay)))
		{
#ifdef BEMUSED_SERIES_60
        iTabGroup->SetActiveTabByIndex(1);
        TRAPD(ignore, ActivateLocalViewL(TUid::Uid(iTabGroup->TabIdFromIndex(1))));
#else
		TRAPD(ignore, iMainView->TabScreen()->SetCurrentTabL(1));
#endif
		}

	iCurrentFile = aFileNode;
	}

void CBemusedAppUi::AddFilesToPlaylist(CDirTreeNode* aNode)
	{
	if (aNode) 
		{
		if (aNode->iFirstChild)
			{
			// more subchilds
			AddFilesToPlaylist(aNode->iFirstChild);
			
			if (aNode->iFirstChild)
				{
				// leaved this method and is a dir
				if (aNode->iNextSibling)
					AddFilesToPlaylist(aNode->iNextSibling);
				}
			}		
		else
			{
			// collect files in this directory
			
			if (!aNode->iFirstChild)
				{ // no directory
				HBufC8* fileName;
				fileName = aNode->iFileName->Alloc();
				
				if ((fileName)&&(aNode->iFileType!=1))
					{
					iPlaylist.Append(fileName);
					}
				aNode = aNode->iNextSibling;
				
				while (aNode)
					{
					fileName = aNode->iFileName->Alloc();
					if (fileName)
						iPlaylist.Append(fileName);
					aNode = aNode->iNextSibling;
					}
				}
			}
		}
	}

void CBemusedAppUi::DownloadFile(const TDesC8& aFileName)
	{
	iBluetoothHandler->RequestFileInfo(aFileName);
	}

TInt CBemusedAppUi::SongTimerCallback(TAny* aPtr)
	{
	CBemusedAppUi* appUi = (CBemusedAppUi*) aPtr;
	appUi->HandleSongTimer();
	return 0;
	}

void CBemusedAppUi::HandleSongTimer()
	{
	if (iSongState == ESongPlaying)
		{
		iCurrentTime++;
		if ((iCurrentTime >= iSongLength) && (iSongLength > 0))
			{
			if (iPlaylist.Count() > 0)
				{
				if (iPlaylistIndex < (iPlaylist.Count() - 1))
					iPlaylistIndex++;
				else if (iRepeat)
					iPlaylistIndex = 0;
				}

			iBluetoothHandler->RequestInfo();

			// Stop info being continually requested.
			iSongLength = -1;
			}

		if (!ControlView()->ControlContainer() ||
			!ControlView()->ControlContainer()->CheckFastForwardAndRewind())
			{
			DrawControlView(KBemusedDrawTrackName | KBemusedDrawTrackTime);
			}
		}

#ifdef BEMUSED_SERIES_60
	if (BrowseView()->BrowseContainer())
		BrowseView()->BrowseContainer()->ScrollSelectedSong();
	else if (PlaylistView()->IsFocused())
		PlaylistView()->ScrollSelectedSong();
#endif
	}

void CBemusedAppUi::IncomingCall(const TDesC& /*aName*/)
	{
	switch (iSettings.iActionOnIncomingCall)
		{
	case TBemusedSettings::EPause:
		if (iSongState == ESongPlaying)
			iBluetoothHandler->RequestPause();
		break;

	case TBemusedSettings::EStop:
		if (iSettings.iActionOnStopping == EFadeOut)
			iBluetoothHandler->RequestStop(EFadeOut);
		else
			iBluetoothHandler->RequestStop(EStopImmediately);
		break;

	case TBemusedSettings::EMute:
		iCurrentVolume = 0;
		iBluetoothHandler->RequestSetVolume(iCurrentVolume);
		break;

	default:
		break;
		}

	iIncomingCallListener->StartListening();
	}

TInt CBemusedAppUi::ReloadSkin()
	{
	TInt err = KErrNone;

#ifdef BEMUSED_SERIES_60
	TRAP(err, iControlView->SkinHandler()->LoadSkinL());
#endif

	return err;
	}

TFileName& CBemusedAppUi::SkinPathL()
	{
	if (iSkinPath.Length() == 0)
		{
		_LIT(KSkins, "skins");
		TFileName app = iEikEnv->EikAppUi()->Application()->AppFullName();
		TParsePtr ptr(app);
		iSkinPath.Copy(ptr.DriveAndPath());
		iSkinPath.Append(KSkins);
		}
	return iSkinPath;
	}

void CBemusedAppUi::BuildIniPath(TFileName & aIniFileName)
	{
	// Get the application name with full path
	TFileName app = iEikEnv->EikAppUi()->Application()->AppFullName();

	// Replace extension with ini
	TParsePtr parse(app);
	aIniFileName.Copy(parse.DriveAndPath());
	aIniFileName.Append(parse.Name());
	_LIT(KIni, ".ini");
	aIniFileName.Append(KIni);
	}

void CBemusedAppUi::SelectInPlaylist(TInt aIndex)
	{
	iPlaylistIndex = aIndex;
	iBluetoothHandler->RequestSelectInPlaylist(aIndex);
	}

CDirTreeNode* CBemusedAppUi::CurrentDirectory()
	{
	return iCurrentDirectory;
	}

void CBemusedAppUi::SetCurrentDirectory(CDirTreeNode* aDirectory)
	{
	iCurrentDirectory = aDirectory;
	}

TBemusedSettings& CBemusedAppUi::Settings()
	{
	return iSettings;
	}

TUint CBemusedAppUi::DrawFlags()
	{
	return iDrawFlags;
	}

void CBemusedAppUi::SetDrawFlags(TUint aFlags)
	{
	iDrawFlags = aFlags;
	}

CBluetoothHandler* CBemusedAppUi::BluetoothHandler()
	{
	return iBluetoothHandler;
	}

TSongState CBemusedAppUi::SongState()
	{
	return iSongState;
	}

RPointerArray<HBufC8>& CBemusedAppUi::Playlist()
	{
	return iPlaylist;
	}

TInt& CBemusedAppUi::PlaylistIndex()
	{
	return iPlaylistIndex;
	}

void CBemusedAppUi::SetPlaylistIndex(TInt aIndex)
	{
	iPlaylistIndex = aIndex;
	}

TInt CBemusedAppUi::CurrentTime()
	{
	return iCurrentTime;
	}

TInt CBemusedAppUi::SongLength()
	{
	return iSongLength;
	}

TBool CBemusedAppUi::Shuffle()
	{
	return iShuffle;
	}

TBool CBemusedAppUi::Repeat()
	{
	return iRepeat;
	}

TInt CBemusedAppUi::CurrentVolume()
	{
	return iCurrentVolume;
	}

const TDesC& CBemusedAppUi::SongNameGuess()
	{
	if (iSongNameGuess.Length() > 0)
		return iSongNameGuess;
	else
		return KBemusedName;
	}

void CBemusedAppUi::SetSkinAuthor(const TDesC8& aAuthor)
	{
	iSkinAuthor = aAuthor;
	}

void CBemusedAppUi::SetSkinName(const TDesC& aName)
	{
	iSkinName = aName;
	}

void CBemusedAppUi::SetDrawPending(TBool aPending)
	{
	iDrawPending = aPending;
	}

// This function ensures that drawing of the control view
// is efficient and flicker-free.
void CBemusedAppUi::DrawControlView(TUint aDrawFlags)
	{
	if (!iDrawPending)
		{
		iDrawFlags = aDrawFlags;
		iDrawPending = ETrue;

#ifndef BEMUSED_SERIES_60
		// Check whether we're in flip-closed mode.
		if (iActiveView == KFlipClosedViewId)
			iFlipClosedView->AppInitiatedDraw();
		else if (ControlView()->ControlContainer())
			ControlView()->ControlContainer()->AppInitiatedDraw();
#else
		if (ControlView()->ControlContainer())
			ControlView()->ControlContainer()->AppInitiatedDraw();
#endif
		}
	else
		{
		iDrawFlags |= aDrawFlags;
		}
	}

void CBemusedAppUi::SetVolume(TInt aVolume)
	{
	iCurrentVolume = (TUint8) aVolume;
	iBluetoothHandler->RequestSetVolume(iCurrentVolume);
	}

void CBemusedAppUi::ReceivedVolume(TInt aVolume)
	{
	iCurrentVolume = (TUint8) aVolume;
	DrawControlView(KBemusedDrawVolume);
	}

CCoeEnv* CBemusedAppUi::CoeEnv()
	{
	return iEikEnv;
	}

void CBemusedAppUi::GetPlaylist()
	{
	iPlaylist.ResetAndDestroy();
	iPlaylistIndex = 0;
	iBluetoothHandler->RequestGetPlaylist();
	}


////////////////////////////////////////////////////////////////////////////////
// Series 60-only code below this point.
////////////////////////////////////////////////////////////////////////////////

#ifdef BEMUSED_SERIES_60

void CBemusedAppUi::ConstructL()
    {
    BaseConstructL();

    // Show tabs for main views from resources
    CEikStatusPane* sp = StatusPane();

    // Fetch pointer to the default navi pane control
    iNaviPane = (CAknNavigationControlContainer*)sp->ControlL( 
        TUid::Uid(EEikStatusPaneUidNavi));

    // Tabgroup has been read from resource and pushed to the navi pane. 
    // Get pointer to the navigation decorator with the ResourceDecorator() function. 
    // Application owns the decorator and it has responsibility to delete the object.
    iDecoratedTabGroup = iNaviPane->ResourceDecorator();
    if (iDecoratedTabGroup)
        iTabGroup = (CAknTabGroup*) iDecoratedTabGroup->DecoratedControl();

	// Get a reference to the EIKON environment.
	iEikEnv = CEikonEnv::Static();

	// Load our settings from the INI file.
	iSettings.iSongBrowserFont = TBemusedSettings::EMedium;
	iCurrentVolume = KStartVolume;
	LoadSettings();
	
	// Start out by needing to redraw the entire control view.
	iDrawFlags = KBemusedDrawEverything;

	// Create each of the views in turn.

    iBrowseView = new (ELeave) CBemusedBrowseView;

    CleanupStack::PushL(iBrowseView);
    iBrowseView->ConstructL();
    AddViewL(iBrowseView);      // transfer ownership to CAknViewAppUi
    CleanupStack::Pop();    // iBrowseView

	// This one must be created before the control view.

	iPlaylistView = new (ELeave) CBemusedPlaylistView;

    CleanupStack::PushL(iPlaylistView);
    iPlaylistView->ConstructL();
    AddViewL(iPlaylistView);      // transfer ownership to CAknViewAppUi
    CleanupStack::Pop();    // iPlaylistView

    iControlView = new (ELeave) CBemusedControlView;

    CleanupStack::PushL(iControlView);
    iControlView->ConstructL();
    AddViewL(iControlView);      // transfer ownership to CAknViewAppUi
    CleanupStack::Pop();    // iControlView

	iDisplaySettingsView = new (ELeave) CBemusedDisplaySettingsView;

	CleanupStack::PushL(iDisplaySettingsView);
	iDisplaySettingsView->ConstructL();
	AddViewL(iDisplaySettingsView);	// transfer ownership to CAknViewAppUi
	CleanupStack::Pop(); // iDisplaySettingsView

	iBehaviourSettingsView = new (ELeave) CBemusedBehaviourSettingsView;

	CleanupStack::PushL(iBehaviourSettingsView);
	iBehaviourSettingsView->ConstructL();
	AddViewL(iBehaviourSettingsView);	// transfer ownership to CAknViewAppUi
	CleanupStack::Pop(); // iBehaviourSettingsView

    SetDefaultViewL(*iBrowseView);

	if (iSettings.iConnectToPc == TBemusedSettings::EAutoToSamePc)
		{
		iBluetoothHandler = CBluetoothHandler::NewL(this, iSettings.iAutoConnectAddress,
			iSettings.iAutoConnectPort);
		}
	else
		{
		TBTDevAddr zeroAddr;
		iBluetoothHandler = CBluetoothHandler::NewL(this, zeroAddr, 0);
		}

	iSongTimer = CPeriodic::NewL(EPriorityNormal);

	iIncomingCallListener = CIncomingCallListener::NewL(*this);
	iIncomingCallListener->StartListening();
    }

TKeyResponse CBemusedAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
    {
    if ((iTabGroup == NULL) || (aType != EEventKey))
        return EKeyWasNotConsumed;

//	TBuf<256> formatBuf;
//	formatBuf.Format(_L("Code: %d Scancode: %d"), aKeyEvent.iCode, aKeyEvent.iScanCode);
//	CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

    TInt active = iTabGroup->ActiveTabIndex();
    TInt count = iTabGroup->TabCount();

	TKeyResponse response = EKeyWasNotConsumed;

	if (iPlaylistView->IsFocused())
		{
		response = iPlaylistView->OfferKeyEventL(aKeyEvent, aType);
		if (response == EKeyWasConsumed)
			return response;
		}

	// If the settings view is currently selected, pass the key event to it.
	if ((iDisplaySettingsView->SettingsContainer()) &&
		(iDisplaySettingsView->SettingsContainer()->IsFocused()))
		{
		response = iDisplaySettingsView->SettingsContainer()->OfferKeyEventL(aKeyEvent, aType);
		if (response == EKeyWasConsumed)
			return response;
		}

	// If the behaviour settings view is currently selected, pass the key event to it.
	if ((iBehaviourSettingsView->SettingsContainer()) &&
		(iBehaviourSettingsView->SettingsContainer()->IsFocused()))
		{
		response = iBehaviourSettingsView->SettingsContainer()->OfferKeyEventL(aKeyEvent, aType);
		if (response == EKeyWasConsumed)
			return response;
		}

	// If the control view is currently selected, give it a chance to take this event.
	if ((iControlView->ControlContainer()) && (iControlView->ControlContainer()->IsFocused()))
		{
		response = iControlView->ControlContainer()->OfferKeyEventL(aKeyEvent, aType);
		if (response == EKeyWasConsumed)
			return response;
		}

    switch (aKeyEvent.iCode)
        {
		case EKeyDevice3:
			{
			// The user has pressed the joystick, so let's select a file in the list box.
			if (iBrowseView->BrowseContainer())
				iBrowseView->BrowseContainer()->FileSelectedL();
			}
			break;

        case EKeyLeftArrow:
			// We will get here if the joystick has been moved left when the user is either
			// in the browse view, or in the settings view, or at the left-hand edge of the
			// control view.

			// Just ignore it if we're doing something.
			if (iBluetoothHandler->IsActive())
				break;

            if ((iSettings.iChangeView != TBemusedSettings::EUsingAbc) &&
				(iSettings.iChangeView != TBemusedSettings::EAbcAutoOnPlay) &&
				(active > 0))
                {
				if (iControlView->Skin().iValid)
					active--;
				else
					active = 0;
                iTabGroup->SetActiveTabByIndex(active);
                ActivateLocalViewL(TUid::Uid(iTabGroup->TabIdFromIndex(active)));
                }
			else
				{
				if (iBrowseView->BrowseContainer())
					iBrowseView->BrowseContainer()->UpOneLevelL();
				}
            break;

        case EKeyRightArrow:

			// Just ignore it if we're doing something.
			if (iBluetoothHandler->IsActive())
				break;

            if ((iSettings.iChangeView != TBemusedSettings::EUsingAbc) &&
				(iSettings.iChangeView != TBemusedSettings::EAbcAutoOnPlay) &&
				((active + 1) < count))
                {
				if (iControlView->Skin().iValid)
					active++;
				else
					active = (count - 1);
                iTabGroup->SetActiveTabByIndex(active);

				// Are we moving to the playlist view?
				if (active == 2)
					GetPlaylist();

                ActivateLocalViewL(TUid::Uid(iTabGroup->TabIdFromIndex(active)));
                }
			else
				{
				if (iBrowseView->BrowseContainer())
					iBrowseView->BrowseContainer()->FileSelectedL();
				}
            break;

		case EKeyUpArrow:
			if (active == 0)
				return EKeyWasNotConsumed;

#ifdef POWERPOINT_ONLY
			if (iPlaylistIndex < (iPlaylist.Count()-1))
				iPlaylistIndex++;
			iBluetoothHandler->RequestNextTrack();
#else
			if (iCurrentVolume < (256 - KVolumeStep))
				iCurrentVolume += KVolumeStep;
			else
				iCurrentVolume = 255;

			iBluetoothHandler->RequestSetVolume(iCurrentVolume);

//			if (iControlView->Container())
//				{
//				iDrawFlags = KBemusedDrawVolume;
//				iControlView->Container()->DrawDeferred();
//				}
			DrawControlView(KBemusedDrawVolume);
#endif
			break;

		case EKeyDownArrow:
			if (active == 0)
				return EKeyWasNotConsumed;

#ifdef POWERPOINT_ONLY
			if (iPlaylistIndex > 0)
				iPlaylistIndex--;
			iBluetoothHandler->RequestPreviousTrack();
#else
			if (iCurrentVolume >= KVolumeStep)
				iCurrentVolume -= KVolumeStep;
			else
				iCurrentVolume = 0;

			iBluetoothHandler->RequestSetVolume(iCurrentVolume);

//			if (iControlView->Container())
//				{
//				iDrawFlags = KBemusedDrawVolume;
//				iControlView->Container()->DrawDeferred();
//				}
			DrawControlView(KBemusedDrawVolume);
#endif
			break;

		case '7': // 7 = P for "add to Playlist"
			if (iBrowseView->BrowseContainer())
				iBrowseView->BrowseContainer()->PlaySelectedFileL(ETrue);
			break;

		case '3': // 3 = F for "Find song"
			if (iBrowseView->BrowseContainer())
				iBrowseView->BrowseContainer()->FindSongL(EFalse);
			break;

		case '6': // 6 = N for "find Next"
			if (iBrowseView->BrowseContainer())
				iBrowseView->BrowseContainer()->FindSongL(ETrue);
			break;

		case 63499: // the "abc" key

			// Just ignore it if we're doing something.
			if (iBluetoothHandler->IsActive())
				break;

            if ((active + 1) < count)
                {
				if (iControlView->Skin().iValid)
					active++;
				else
					active = (count - 1);

				// Are we moving to the playlist view?
				if (active == 2)
					GetPlaylist();
                }
			else
				{
				active = 0;
				}

            iTabGroup->SetActiveTabByIndex(active);
            ActivateLocalViewL(TUid::Uid(iTabGroup->TabIdFromIndex(active)));
			break;

        default:
            return EKeyWasNotConsumed;
            break;
        }

	return EKeyWasConsumed;
	}

void CBemusedAppUi::DisplayDetailedInfoDialogL(TInt aSampleRate, TInt aBitRate, TInt aNumChannels)
	{
	CEikTextListBox* list = 
					 new(ELeave) CAknSinglePopupMenuStyleListBox;

	CleanupStack::PushL(list);
	CAknPopupList* popupList = CAknPopupList::NewL(
								list, R_AVKON_SOFTKEYS_BACK,
								AknPopupLayouts::EMenuWindow);

	CleanupStack::PushL(popupList);
	list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
	list->CreateScrollBarFrameL(ETrue);
	list->ScrollBarFrame()->SetScrollBarVisibilityL(
								CEikScrollBarFrame::EOff,
								CEikScrollBarFrame::EAuto);

	// Set Item text
	CDesCArrayFlat* items = new (ELeave) CDesCArrayFlat(5);
	CleanupStack::PushL(items);

	TBuf8<256> formatBuf8;
	TBuf<256> formatBuf;
	TBuf<256> resourceBuf;

	if ((aSampleRate > 0) && (aBitRate > 0) && (aNumChannels > 0))
		{
		iEikEnv->ReadResource(resourceBuf, R_BITRATE_DISPLAY);

		formatBuf.Format(resourceBuf, aBitRate);
		items->AppendL(formatBuf);

		_LIT(KStereo, "stereo");
		_LIT(KMono, "mono");

		iEikEnv->ReadResource(resourceBuf, R_SAMPLERATE_DISPLAY);

		formatBuf.Format(resourceBuf, aSampleRate, (aNumChannels == 1) ? &KMono : &KStereo);
		items->AppendL(formatBuf);
		}

	iEikEnv->ReadResource(resourceBuf, R_SKIN_DISPLAY);
	formatBuf.Format(resourceBuf, &iSkinName);

	items->AppendL(formatBuf);

	iEikEnv->ReadResource(formatBuf, R_UNKNOWN_AUTHOR);
	
	if (iSkinAuthor != KNullDesC8)
		{
		formatBuf8.Format(_L8(" (%S)"), &iSkinAuthor);
		formatBuf.Copy(formatBuf8);
		}

	items->AppendL(formatBuf);

	// Set listitems.
	CTextListBoxModel* model = list->Model();
	model->SetItemTextArray(items);
	model->SetOwnershipType(ELbmOwnsItemArray);
	CleanupStack::Pop();

	list->ClearSelection();

	iEikEnv->ReadResource(resourceBuf, R_TRACK_INFO);
	popupList->SetTitleL(resourceBuf);

	// Show popup list.
	popupList->ExecuteLD();

	CleanupStack::Pop();            // popuplist
	CleanupStack::PopAndDestroy();  // list
	}

CBemusedControlView* CBemusedAppUi::ControlView()
	{
	return iControlView;
	}

CBemusedBrowseView* CBemusedAppUi::BrowseView()
	{
	return iBrowseView;
	}

CBemusedPlaylistView* CBemusedAppUi::PlaylistView()
	{
	return iPlaylistView;
	}


////////////////////////////////////////////////////////////////////////////////
// UIQ-only code below this point.
////////////////////////////////////////////////////////////////////////////////

#else

void CBemusedAppUi::ConstructL()
	{
	BaseConstructL();

	// Get a reference to the EIKON environment.
	iEikEnv = CEikonEnv::Static();

	// Load our settings from the INI file.
	iSettings.iSongBrowserFont = TBemusedSettings::EMedium;
	iCurrentVolume = KStartVolume;
	LoadSettings();

	// Force the display type to full screen.
	iSettings.iDisplayType = TBemusedSettings::EFullScreen;
	
	// Start out by needing to redraw the entire control view.
	iDrawFlags = KBemusedDrawEverything;

	iMainView = new (ELeave) CBemusedMainView;
	RegisterViewL(*iMainView);

	iFlipClosedView = new (ELeave) CBemusedFlipClosedView(this);
	RegisterViewL(*iFlipClosedView);

	iMainView->ConstructL(this);
	iFlipClosedView->ConstructL();

	ActivateViewL(TVwsViewId(KUidBemused, KMainViewId));

	iSongTimer = CPeriodic::NewL(EPriorityNormal);

	if (iSettings.iConnectToPc == TBemusedSettings::EAutoToSamePc)
		{
		iBluetoothHandler = CBluetoothHandler::NewL(this, iSettings.iAutoConnectAddress,
			iSettings.iAutoConnectPort);
		}
	else
		{
		TBTDevAddr zeroAddr;
		iBluetoothHandler = CBluetoothHandler::NewL(this, zeroAddr, 0);
		}

//	iIncomingCallListener = CIncomingCallListener::NewL(*this);

//	iEikEnv->InfoWinL(_L("Created call listener"), KNullDesC);

//	iIncomingCallListener->StartListening();

//	iEikEnv->InfoWinL(_L("Started listening"), KNullDesC);
	}

TKeyResponse CBemusedAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
    {
	if (iFlipClosedView->IsVisible())
		return iFlipClosedView->OfferKeyEventL(aKeyEvent, aType);

	if (aType != EEventKey)
		return EKeyWasNotConsumed;

	if (iMainView->TabIndex() == 0)
		{
		switch (aKeyEvent.iScanCode)
			{
		case EStdKeyDevice1: // jog dial up
			iMainView->BrowseContainer()->SelectPreviousFile();
			break;

		case EStdKeyDevice2: // jog dial down
			iMainView->BrowseContainer()->SelectNextFile();
			break;

		case EStdKeyDevice8: // jog dial pressed
			iMainView->BrowseContainer()->FileSelectedL();
			break;

		default:
			break;
			}
		}
	else if (iMainView->TabIndex() == 1)
		{
		switch (aKeyEvent.iScanCode)
			{
		case EStdKeyDevice1: // jog dial up
#ifdef POWERPOINT_ONLY
			if (iPlaylistIndex < (iPlaylist.Count()-1))
				iPlaylistIndex++;
			iBluetoothHandler->RequestNextTrack();
#else
			if (iCurrentVolume < (256 - KVolumeStep))
				iCurrentVolume += KVolumeStep;
			else
				iCurrentVolume = 255;

			iBluetoothHandler->RequestSetVolume(iCurrentVolume);
			DrawControlView(KBemusedDrawVolume);
#endif
			break;

		case EStdKeyDevice2: // jog dial down
#ifdef POWERPOINT_ONLY
			if (iPlaylistIndex > 0)
				iPlaylistIndex--;
			iBluetoothHandler->RequestPreviousTrack();
#else
			if (iCurrentVolume >= KVolumeStep)
				iCurrentVolume -= KVolumeStep;
			else
				iCurrentVolume = 0;

			iBluetoothHandler->RequestSetVolume(iCurrentVolume);
			DrawControlView(KBemusedDrawVolume);
#endif
			break;

		case EStdKeyDevice8: // jog dial pressed
			if (iPlaylistIndex < (iPlaylist.Count()-1))
				iPlaylistIndex++;
			iBluetoothHandler->RequestNextTrack();
			break;

		default:
			return EKeyWasNotConsumed;
			break;
			}
		}
	else if (iMainView->TabIndex() == 2)
		{
		switch (aKeyEvent.iScanCode)
			{
		case EStdKeyDevice1: // jog dial up
			iMainView->SelectPreviousFileInPlaylist();
			break;

		case EStdKeyDevice2: // jog dial down
			iMainView->SelectNextFileInPlaylist();
			break;

		case EStdKeyDevice8: // jog dial pressed
			iMainView->PlaylistFileSelected();
			break;

		default:
			break;
			}
		}

	return EKeyWasConsumed;
	}

void CBemusedAppUi::DisplayDetailedInfoDialogL(TInt aSampleRate, TInt aBitRate, TInt aNumChannels)
	{
	TBuf8<128> formatBuf8;
	TBuf<128> formatBuf;
	TBuf<128> resourceBuf;
	TBuf<256> totalBuf;

	if ((aSampleRate > 0) && (aBitRate > 0) && (aNumChannels > 0))
		{
		iEikEnv->ReadResource(resourceBuf, R_BITRATE_DISPLAY);

		formatBuf.Format(resourceBuf, aBitRate);
		totalBuf.Append(formatBuf);

		_LIT(KStereo, "stereo");
		_LIT(KMono, "mono");

		iEikEnv->ReadResource(resourceBuf, R_SAMPLERATE_DISPLAY);

		formatBuf.Format(resourceBuf, aSampleRate, (aNumChannels == 1) ? &KMono : &KStereo);
		totalBuf.Append(formatBuf);
		}

	iEikEnv->ReadResource(resourceBuf, R_SKIN_DISPLAY);
	formatBuf.Format(resourceBuf, &iSkinName);

	totalBuf.Append(_L("\n"));
	totalBuf.Append(formatBuf);

	iEikEnv->ReadResource(formatBuf, R_UNKNOWN_AUTHOR);
	
	if (iSkinAuthor != KNullDesC8)
		{
		formatBuf8.Format(_L8(" (by %S)"), &iSkinAuthor);
		formatBuf.Copy(formatBuf8);
		}

	totalBuf.Append(formatBuf);

	iEikEnv->ReadResource(resourceBuf, R_TRACK_INFO);

	iEikEnv->InfoWinL(resourceBuf, totalBuf);
	}

CBemusedMainView* CBemusedAppUi::ControlView()
	{
	return iMainView;
	}

CBemusedMainView* CBemusedAppUi::BrowseView()
	{
	return iMainView;
	}

CBemusedMainView* CBemusedAppUi::PlaylistView()
	{
	return iMainView;
	}

void CBemusedAppUi::SetActiveView(TUid aViewUid)
	{
	iActiveView = aViewUid;
	}

#endif // BEMUSED_SERIES_60

// End of File  
