/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <gdi.h>
#include <eiktxlbm.h>

#ifdef BEMUSED_SERIES_60
  #include <AknUtils.h>
  #include <Bemused.rsg>
#else
  #include <Bemused.rsg>
#endif

#include "BemusedListBox.h"
#include "BemusedAppUi.h"

CBemusedListBoxView::CBemusedListBoxView()
	{
	}

void CBemusedListBoxView::DrawEmptyList(const TRect &aClientRect) const
	{
#ifdef BEMUSED_SERIES_60
	AknDraw::DrawEmptyList(aClientRect, *iGc, iListEmptyText->Des());
#else
	iItemDrawer->ClearRect(aClientRect);
#endif
	}

void CBemusedListBoxView::Draw(const TRect* aClipRect) const
	{
	if (iModel->NumberOfItems() > 0)
		CListBoxView::Draw(aClipRect);
	else
		DrawEmptyList(iViewRect);
	}

////////////////////////////////////////////////////////////////////////////////

CBemusedListItemDrawer::CBemusedListItemDrawer(MTextListBoxModel* aTextListBoxModel,
											   const CFont* aFont)
	: CTextListItemDrawer(aTextListBoxModel, aFont), iPersistentFont(aFont)
	{
	}

void CBemusedListItemDrawer::DrawActualItem(TInt aItemIndex, const TRect& aActualItemRect,
											TBool aItemIsCurrent, TBool /*aViewIsEmphasized*/,
											TBool, TBool /*aItemIsSelected*/) const
	{
	TRgb textColour;
	TRgb bgColour;

	if (aItemIsCurrent)
		{
		textColour = iHighlightForegroundColour;
		bgColour = iHighlightBackgroundColour;
		}
	else
		{
		textColour = iTextForegroundColour;
		bgColour = iTextBackgroundColour;
		}

	// We need to set the font every time, because AknDraw::DrawEmptyList() resets it to the
	// standard size.
	Gc()->UseFont(iPersistentFont);

	// Set the item's background.
	Gc()->SetBrushStyle(CGraphicsContext::ESolidBrush);
	Gc()->SetBrushColor(bgColour);
	Gc()->SetPenColor(bgColour);

	// Draw the item's text.
	Gc()->SetPenColor(textColour);

	// If this item is currently being scrolled, then select some of it. Otherwise, use the entire
	// string. Some spaces are always prepended as well.
	TBuf<256> displayBuf;
	if (aItemIsCurrent)
		displayBuf = iModel->ItemText(aItemIndex).Mid(iScrollOffset);
	else
		displayBuf = iModel->ItemText(aItemIndex);
	displayBuf.Insert(0, _L("  "));

	Gc()->DrawText(displayBuf, aActualItemRect,
		(aActualItemRect.Height() / 2) + (iFont->AscentInPixels() / 2));

	// Set the pen to the colour of the background, so there isn't a visible frame around the item.
	Gc()->SetPenColor(bgColour);
	}

void CBemusedListItemDrawer::SetColours(const TRgb& aTextForeground, const TRgb& aTextBackground,
	const TRgb& aHighlightForeground, const TRgb& aHighlightBackground)
	{
	iTextForegroundColour = aTextForeground;
	iTextBackgroundColour = aTextBackground;
	iHighlightForegroundColour = aHighlightForeground;
	iHighlightBackgroundColour = aHighlightBackground;

	iBackColor = iTextBackgroundColour;
	}

TBool CBemusedListItemDrawer::ScrollSong(TInt aItemIndex)
	{
	TBuf<256> displayBuf;

	displayBuf = iModel->ItemText(aItemIndex);
	displayBuf.Insert(0, _L("  "));

	// Check that the track name fits into the space available. If not, we scroll it. There's
	// a small delay at the beginning and end of scrolling.

	TInt trackNameChars = iPersistentFont->TextCount(displayBuf, KFullScreenWidth);

	if (trackNameChars < displayBuf.Length())
		{
		iScrollOffset += 2;

		if (iScrollOffset > (displayBuf.Length() - trackNameChars + 1))
			iScrollOffset = 0;

		return ETrue;
		}

	return EFalse;
	}

void CBemusedListItemDrawer::ResetScrolling()
	{
	iScrollOffset = 0;
	}

////////////////////////////////////////////////////////////////////////////////

CBemusedListBox::CBemusedListBox(const CFont* aFont)
	{
	iFont = aFont;
	}

void CBemusedListBox::CreateItemDrawerL()
	{
	iItemDrawer = new (ELeave) CBemusedListItemDrawer(Model(), iFont);
	}

CListBoxView* CBemusedListBox::MakeViewClassInstanceL()
	{
	return new (ELeave) CBemusedListBoxView();
	}

// End of File  
