/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eikenv.h>

#include "IncomingCallListener.h"

CIncomingCallListener::CIncomingCallListener(MIncomingCallListener& aListener)
	: CActive(EPriorityStandard), iListener(aListener)
	{
	CActiveScheduler::Add(this);
	}

CIncomingCallListener* CIncomingCallListener::NewL(MIncomingCallListener& aListener)
	{
	CIncomingCallListener* self = new (ELeave) CIncomingCallListener(aListener);

	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(); // self

	return self;
	}

void CIncomingCallListener::ConstructL()
	{
//	CEikonEnv::Static()->InfoWinL(_L("About to connect to etel"), KNullDesC);

	User::LeaveIfError(iTelServer.Connect());

//	CEikonEnv::Static()->InfoWinL(_L("Connected to etel"), KNullDesC);

	RTelServer::TPhoneInfo phoneInfo;
	User::LeaveIfError(iTelServer.GetPhoneInfo(0, phoneInfo));

//		CEikonEnv::Static()->InfoWinL(_L("got phone info"), KNullDesC);

	User::LeaveIfError(iPhone.Open(iTelServer, phoneInfo.iName));

//		CEikonEnv::Static()->InfoWinL(_L("opened phone"), KNullDesC);

	RPhone::TLineInfo lineInfo;
	User::LeaveIfError(iPhone.GetLineInfo(0, lineInfo));

//		CEikonEnv::Static()->InfoWinL(_L("got line info"), KNullDesC);

	// Open phone "Calypso", line "VoiceLine1".

	User::LeaveIfError(iLine.Open(iPhone, lineInfo.iName));

//		CEikonEnv::Static()->InfoWinL(_L("opened line"), KNullDesC);
	}

CIncomingCallListener::~CIncomingCallListener()
	{
	Cancel();

	iLine.Close();
	iPhone.Close();
	iTelServer.Close();
	}

void CIncomingCallListener::StartListening()
	{
	iLine.NotifyIncomingCall(iStatus, iCallName);
	SetActive();
	}

void CIncomingCallListener::RunL()
	{
	User::LeaveIfError(iStatus.Int());

	iListener.IncomingCall(iCallName);
	}

TInt CIncomingCallListener::RunError(TInt /*aError*/)
	{
	return KErrNone;
	}

void CIncomingCallListener::DoCancel()
	{
	iLine.NotifyIncomingCallCancel();
	}