/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <aknviewappui.h>
#include <avkon.hrh>
#include <Bemused.rsg>

#include "Bemused.hrh"
#include "BemusedBehaviourSettingsView.h"
#include "BemusedBehaviourSettingsContainer.h" 

void CBemusedBehaviourSettingsView::ConstructL()
    {
    BaseConstructL(R_BEMUSED_BEHAVIOUR_SETTINGS_VIEW);
    }

CBemusedBehaviourSettingsView::~CBemusedBehaviourSettingsView()
    {
    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);

    delete iContainer;
    }

TUid CBemusedBehaviourSettingsView::Id() const
    {
    return KBehaviourSettingsViewId;
    }

void CBemusedBehaviourSettingsView::HandleCommandL(TInt aCommand)
    {   
    switch (aCommand)
        {
        case EAknSoftkeyOk:
            break;

        case EAknSoftkeyBack:
            AppUi()->HandleCommandL(EBemusedCmdExitBehaviourSettings);
            break;

        default:
            AppUi()->HandleCommandL(aCommand);
            break;
        }
    }

// ---------------------------------------------------------
// CBemusedSettingsView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CBemusedBehaviourSettingsView::HandleClientRectChange()
    {
    if (iContainer)
        iContainer->SetRect(ClientRect());
    }

// ---------------------------------------------------------
// CBemusedSettingsView::DoActivateL(...)
// ?implementation_description
// ---------------------------------------------------------
//
void CBemusedBehaviourSettingsView::DoActivateL(
   const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/,
   const TDesC8& /*aCustomMessage*/)
    {
	Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_BACK);

    if (!iContainer)
        {
		CBemusedAppUi* appUi = (CBemusedAppUi*) AppUi();

        iContainer = new (ELeave) CBemusedBehaviourSettingsContainer;
        iContainer->SetMopParent(this);
        iContainer->ConstructL(appUi, this, ClientRect());
        AppUi()->AddToStackL(*this, iContainer);
        } 
   }

void CBemusedBehaviourSettingsView::DoDeactivate()
    {
	// Set the CBA back to the default.
	Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_EXIT);
	Cba()->DrawDeferred();

    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);
    
    delete iContainer;
    iContainer = NULL;
    }

CBemusedBehaviourSettingsContainer* CBemusedBehaviourSettingsView::SettingsContainer()
	{
	return iContainer;
	}

// End of File

