/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <aknviewappui.h>
#include <avkon.hrh>
#include <Bemused.rsg>
#include "BemusedBrowseView.h"
#include "BemusedControlView.h"
#include "BemusedPlaylistView.h"
#include "BemusedControlContainer.h" 
#include "BemusedAppUi.h"
#include "BemusedSkinHandler.h"

void CBemusedControlView::ConstructL()
    {
    BaseConstructL(R_BEMUSED_CONTROL_VIEW);

	iSkinHandler = CBemusedSkinHandler::NewL((CBemusedAppUi*) AppUi(), iSkin);
	}

CBemusedControlView::~CBemusedControlView()
    {
    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);

	delete iSkinHandler;
    delete iContainer;
    }

TUid CBemusedControlView::Id() const
    {
    return KControlViewId;
    }

void CBemusedControlView::HandleCommandL(TInt aCommand)
    {   
    switch (aCommand)
        {
    case EAknSoftkeyOk:
        break;

    case EAknSoftkeyExit:
        AppUi()->HandleCommandL(EEikCmdExit);
        break;

    default:
        AppUi()->HandleCommandL(aCommand);
        break;
        }
    }

void CBemusedControlView::HandleClientRectChange()
    {
    if (iContainer)
        {
        iContainer->SetRect(ClientRect());
        }
    }

void CBemusedControlView::DoActivateL(
   const TVwsViewId& /*aPrevViewId*/,TUid /*aCustomMessageId*/,
   const TDesC8& /*aCustomMessage*/)
    {
    if (!iContainer)
        {
        iContainer = new (ELeave) CBemusedControlContainer;
        iContainer->SetMopParent(this);

		CBemusedAppUi* appUi = (CBemusedAppUi*) AppUi();

		if (appUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
			iContainer->ConstructL(appUi, this, TRect(0, 0, KFullScreenWidth, KFullScreenHeight));
		else
			iContainer->ConstructL(appUi, this, ClientRect());

        AppUi()->AddToStackL(*this, iContainer);
        }
	}

void CBemusedControlView::DoDeactivate()
    {
    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);
    
    delete iContainer;
    iContainer = NULL;
    }

CBemusedControlContainer* CBemusedControlView::ControlContainer()
	{
	return iContainer;
	}

CBemusedSkinHandler* CBemusedControlView::SkinHandler()
	{
	return iSkinHandler;
	}

TBemusedSkin& CBemusedControlView::Skin()
	{
	return iSkin;
	}

// End of File

