/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <aknviewappui.h>
#include <avkon.hrh>
#include <Bemused.rsg>

#include "Bemused.hrh"
#include "BemusedDisplaySettingsView.h"
#include "BemusedDisplaySettingsContainer.h" 

void CBemusedDisplaySettingsView::ConstructL()
    {
    BaseConstructL(R_BEMUSED_DISPLAY_SETTINGS_VIEW);
    }

CBemusedDisplaySettingsView::~CBemusedDisplaySettingsView()
    {
    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);

    delete iContainer;
    }

TUid CBemusedDisplaySettingsView::Id() const
    {
    return KDisplaySettingsViewId;
    }

void CBemusedDisplaySettingsView::HandleCommandL(TInt aCommand)
    {   
    switch (aCommand)
        {
        case EAknSoftkeyOk:
            break;

        case EAknSoftkeyBack:
			iContainer->iListBox->StoreSettingsL();
            AppUi()->HandleCommandL(EBemusedCmdExitDisplaySettings);
            break;

        default:
            AppUi()->HandleCommandL(aCommand);
            break;
        }
    }

// ---------------------------------------------------------
// CBemusedSettingsView::HandleClientRectChange()
// ---------------------------------------------------------
//
void CBemusedDisplaySettingsView::HandleClientRectChange()
    {
    if (iContainer)
        iContainer->SetRect(ClientRect());
    }

// ---------------------------------------------------------
// CBemusedSettingsView::DoActivateL(...)
// ?implementation_description
// ---------------------------------------------------------
//
void CBemusedDisplaySettingsView::DoActivateL(
   const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/,
   const TDesC8& /*aCustomMessage*/)
    {
	Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_BACK);

    if (!iContainer)
        {
		CBemusedAppUi* appUi = (CBemusedAppUi*) AppUi();

		iOrigDisplayType = (TBemusedSettings::TDisplayType) appUi->Settings().iDisplayType;
		iOrigSkinFileIndex = appUi->Settings().iSkinFileIndex;

        iContainer = new (ELeave) CBemusedDisplaySettingsContainer;
        iContainer->SetMopParent(this);
        iContainer->ConstructL(appUi, this, ClientRect());
        AppUi()->AddToStackL(*this, iContainer);
        } 
   }

void CBemusedDisplaySettingsView::DoDeactivate()
    {
	// Set the CBA back to the default.
	Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_EXIT);
	Cba()->DrawDeferred();

    if (iContainer)
        {
		CBemusedAppUi* appUi = (CBemusedAppUi*) AppUi();

		// Reload the skin if necessary when the user leaves this view.
		if ((appUi->Settings().iDisplayType != iOrigDisplayType) ||
			(appUi->Settings().iSkinFileIndex != iOrigSkinFileIndex))
			{
			appUi->ReloadSkin();
			}

        AppUi()->RemoveFromViewStack( *this, iContainer );
        }
    
    delete iContainer;
    iContainer = NULL;
    }

CBemusedDisplaySettingsContainer* CBemusedDisplaySettingsView::SettingsContainer()
	{
	return iContainer;
	}

// End of File

