/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <aknviewappui.h>
#include <avkon.hrh>
#include <Bemused.rsg>
#include "Bemused.hrh"
#include "AknQueryDialog.h"
#include "BemusedPlaylistView.h"
#include "BemusedAppUi.h"

void CBemusedPlaylistView::ConstructL()
    {
    BaseConstructL(R_BEMUSED_PLAYLIST_VIEW);

	iTextForegroundColour = KRgbBlack;
	iTextBackgroundColour = KRgbWhite;
	iHighlightForegroundColour = KRgbBlack;
	iHighlightBackgroundColour = TRgb(0x80, 0x80, 0xFF);
    }

CBemusedPlaylistView::~CBemusedPlaylistView()
    {
	if (iListBox)
	    AppUi()->RemoveFromViewStack(*this, iListBox);

    delete iListBox;
	delete iListItems;
    }

TUid CBemusedPlaylistView::Id() const
    {
    return KPlaylistViewId;
    }

void CBemusedPlaylistView::HandleCommandL(TInt aCommand)
    {   
    switch (aCommand)
        {
        case EAknSoftkeyOk:
            break;

        case EAknSoftkeyExit:
            AppUi()->HandleCommandL(EEikCmdExit);
            break;

        default:
            AppUi()->HandleCommandL(aCommand);
            break;
        }
    }

void CBemusedPlaylistView::HandleClientRectChange()
    {
    if (iListBox)
        iListBox->SetRect(ClientRect());
    }

void CBemusedPlaylistView::DoActivateL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/,
	const TDesC8& /*aCustomMessage*/)
    {
//	Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_BACK);

    if (!iListBox)
        {
		CBemusedAppUi* appUi = (CBemusedAppUi*) AppUi();
		const CFont* font;

		switch (appUi->Settings().iSongBrowserFont)
			{
		case TBemusedSettings::EHuge:
			font = CEikonEnv::Static()->TitleFont();
			break;
		case TBemusedSettings::ELarge:
			font = CEikonEnv::Static()->NormalFont();
			break;
		case TBemusedSettings::EMedium:
			font = CEikonEnv::Static()->LegendFont();
			break;
		case TBemusedSettings::ESmall:
		default:
			font = CEikonEnv::Static()->DenseFont();
			break;
			}

		iListItems = new (ELeave) CDesC16ArrayFlat(4);

		iListBox = new (ELeave) CBemusedListBox(font);
		iListBox->ConstructL(NULL, EAknListBoxLoopScrolling | CEikListBox::EPaintedSelection);

		iListBox->Model()->SetOwnershipType(ELbmDoesNotOwnItemArray);
		iListBox->Model()->SetItemTextArray(iListItems);

		TBuf<256> resourceBuf;
		((CBemusedAppUi*)AppUi())->CoeEnv()->ReadResource(resourceBuf, R_EMPTY_PLAYLIST);
		iListBox->View()->SetListEmptyTextL(resourceBuf);

		// Set the colour to be used in areas where there aren't any items in the list.
		iListBox->View()->SetBackColor(iTextBackgroundColour);

		// Set the item height to a sensible value that can be used for all fonts.
		iListBox->SetItemHeightL(KItemHeight);

		// Create the scrolling indicators at the bottom of the screen.
		iScrollBarFrame = iListBox->CreateScrollBarFrameL(ETrue);
		iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(
				CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);

		// Set the list box's dimensions based on whether we're full-screen or not.
		if (appUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
			iListBox->SetRect(TRect(0, 0, KFullScreenWidth, KFullScreenHeight));
		else
			iListBox->SetRect(ClientRect());

		iListBox->ActivateL();
		}

	CBemusedListItemDrawer* drawer = (CBemusedListItemDrawer*) iListBox->View()->ItemDrawer();

	drawer->SetColours(iTextForegroundColour, iTextBackgroundColour, iHighlightForegroundColour,
		iHighlightBackgroundColour);

	iListBox->DrawNow();

	iFocused = ETrue;
	}

void CBemusedPlaylistView::DoDeactivate()
    {
    if (iListBox)
		AppUi()->RemoveFromViewStack(*this, iListBox);
    
    delete iListBox;
    iListBox = NULL;

	// Set the CBA back to the default.
	//Cba()->SetCommandSetL(R_AVKON_SOFTKEYS_OPTIONS_EXIT);
	//Cba()->DrawDeferred();

	iFocused = EFalse;
    }

TInt CBemusedPlaylistView::GetSelectedFile()
	{
	TInt index = iListBox->CurrentItemIndex();
	if ((index < 0) || (index >= iListBox->Model()->ItemTextArray()->MdcaCount()))
		return KErrNotFound;

	return index;
	}

void CBemusedPlaylistView::ScrollSelectedSong()
	{
	TInt index = iListBox->CurrentItemIndex();
	if ((index < 0) || (index >= iListBox->Model()->ItemTextArray()->MdcaCount()))
		return;

	if (((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ScrollSong(index))
		iListBox->DrawNow();
	}

void CBemusedPlaylistView::ReadSkinDescriptionL(const TDesC& aFileName)
	{
	// This function reads the bits of the skin description which this
	// view is interested in.

	// Set default values in case we can't open the skin.
	iTextForegroundColour = KRgbBlack;
	iTextBackgroundColour = KRgbWhite;
	iHighlightForegroundColour = KRgbBlack;
	iHighlightBackgroundColour = TRgb(0x80, 0x80, 0xFF);

	RFile file;
	User::LeaveIfError(file.Open(CEikonEnv::Static()->FsSession(),
		aFileName, EFileShareAny | EFileRead));
	CleanupClosePushL(file);

	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));

	HBufC8* fileBuf = HBufC8::NewLC(fileSize);
	TPtr8 fileBufPtr = fileBuf->Des();

	User::LeaveIfError(file.Read(fileBufPtr));

	TLex8 lex(*fileBuf);
	TPtrC8 ptr(lex.NextToken());

	while (ptr != KNullDesC8)
		{
		lex.SkipSpace();

		if (!ptr.CompareF(_L8("textforegroundcolour:")))
			ReadSkinColour(lex, iTextForegroundColour);
		else if (!ptr.CompareF(_L8("textbackgroundcolour:")))
			ReadSkinColour(lex, iTextBackgroundColour);
		else if (!ptr.CompareF(_L8("highlightforegroundcolour:")))
			ReadSkinColour(lex, iHighlightForegroundColour);
		else if (!ptr.CompareF(_L8("highlightbackgroundcolour:")))
			ReadSkinColour(lex, iHighlightBackgroundColour);
		else
			lex.SkipSpace();

		ptr.Set(lex.NextToken());
		}

	CleanupStack::PopAndDestroy(2); // file, fileBuf
	}

void CBemusedPlaylistView::ReadSkinColour(TLex8& aLex, TRgb& aColour)
	{
	TInt colour;

	aLex.Val(colour);
	aColour.SetRed(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetGreen(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetBlue(colour);
	aLex.SkipSpace();
	}

TBool CBemusedPlaylistView::IsFocused()
	{
	return iFocused;
	}

void CBemusedPlaylistView::RefreshPlaylistL(const RPointerArray<HBufC8>& aPlaylist, TInt aIndex)
	{
	((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

	iListItems->Reset();

	for (TInt i = 0; i < aPlaylist.Count(); i++)
		{
		HBufC* fileBuf = HBufC::NewLC(aPlaylist[i]->Length());
		fileBuf->Des().Copy(*aPlaylist[i]);
		iListItems->AppendL(*fileBuf);
		CleanupStack::PopAndDestroy();
		}

	iListBox->Model()->SetItemTextArray(iListItems);
	iListBox->HandleItemAdditionL();

	if ((aIndex >= 0) && (aIndex <= aPlaylist.Count()))
		iListBox->SetCurrentItemIndex(aIndex);

	iListBox->DrawNow();
	}

TKeyResponse CBemusedPlaylistView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	CBemusedAppUi* appUi = ((CBemusedAppUi*) AppUi());
    switch (aKeyEvent.iCode)
        {
		case EKeyDevice3:
			{
			// The user has pressed the joystick, so let's select a file in the list box.
			TInt selection = GetSelectedFile();
			if (selection != KErrNotFound)
				appUi->SelectInPlaylist(selection);
			}
			break;

		case EKeyUpArrow:
		case EKeyDownArrow:
			((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();
			return iListBox->OfferKeyEventL(aKeyEvent, aType);
			break;

		case '0':
			{
			((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

			TInt currentIndex = iListBox->CurrentItemIndex();

			TInt itemsOnScreen = 6;
			if (appUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
				itemsOnScreen = 8;

			if (iListBox->Model()->ItemTextArray()->MdcaCount() > (currentIndex + itemsOnScreen))
				{
				iListBox->SetCurrentItemIndex(currentIndex + itemsOnScreen);
				}
			else
				{
				iListBox->SetCurrentItemIndex(iListBox->Model()->ItemTextArray()->MdcaCount() - 1);
				}
			iListBox->DrawNow();
			}
			break;

		case '2':
			{
			((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

			TInt currentIndex = iListBox->CurrentItemIndex();

			TInt itemsOnScreen = 6;
			if (appUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
				itemsOnScreen = 8;

			if (currentIndex > itemsOnScreen)
				{
				iListBox->SetCurrentItemIndex(currentIndex - itemsOnScreen);
				}
			else
				{
				iListBox->SetCurrentItemIndex(0);
				}
			iListBox->DrawNow();
			}
			break;

		case '3':
			FindSongL(EFalse);
			break;

		case '6':
			FindSongL(ETrue);
			break;

		default:
			return EKeyWasNotConsumed;
			break;
		}

	return EKeyWasConsumed;
	}

void CBemusedPlaylistView::FindSongL(TBool aFindNext)
	{
	TFileName lastSearchedSong = iLastSearchedSong;
	CAknTextQueryDialog* dialog = NULL;

	// Create a dialog to ask the user what to find, if we don't want
	// to just find the next occurrence.
	if (aFindNext)
		{
		if (iLastSearchedSong == KNullDesC)
			return;
		}
	else
		{
		dialog = CAknTextQueryDialog::NewL(iLastSearchedSong);
		CleanupStack::PushL(dialog);

		dialog->SetPredictiveTextInputPermitted(ETrue);
		}

	iResultsFound = 0;

	if (aFindNext || dialog->ExecuteLD(R_FIND_DIALOG))
		{
		// If the user wants to search for something else, don't skip
		// any of the results.
		if (iLastSearchedSong != lastSearchedSong)
			iFindResultsToSkip = 0;

		TInt count = iListBox->Model()->ItemTextArray()->MdcaCount();
		TInt resultsFound = 0;

		for (TInt i = 0; i < count; i++)
			{
			if ((*iListItems)[i].FindF(iLastSearchedSong) != KErrNotFound)
				{
				resultsFound++;

				if (resultsFound > iFindResultsToSkip)
					{
					((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();
					iListBox->SetCurrentItemIndex(i);
					iListBox->DrawNow();

					// If the user searches again, we'll skip this result.
					iFindResultsToSkip++;

					if (!aFindNext)
						CleanupStack::Pop(); // dialog
					return;
					}
				}
			}

		lastSearchedSong.Format(_L("Couldn't find \"%S\""), &iLastSearchedSong);
		CEikonEnv::Static()->InfoWinL(lastSearchedSong, KNullDesC);
		iFindResultsToSkip = 0;
		}

	if (!aFindNext)
		CleanupStack::Pop(); // dialog
	}

// End of File

