/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eikappui.h>
#include <qiktabscreen.h>
#include <barsread.h>
#include <eiktxlbm.h>
#include <eikmenub.h>
#include <eiklabel.h>

#include <Bemused.rsg>
#include "Bemused.hrh"
#include "BemusedFlipClosedView.h"
#include "BemusedAppUi.h"
#include "BemusedBrowseContainer.h" 
#include "BemusedControlContainer.h" 
#include "BemusedApp.h"
#include "BemusedListBox.h"
#include "BemusedFindSongDialog.h"
#include "BemusedMainView.h"

CBemusedFlipClosedView::CBemusedFlipClosedView(CBemusedAppUi* aAppUi)
	: iAppUi(aAppUi), iSkin(aAppUi->ControlView()->Skin())
	{
	iSkinHandler = aAppUi->ControlView()->SkinHandler();
	iFocusIndex = 2;
	}

void CBemusedFlipClosedView::ConstructL()
    {
	CreateWindowL();
	SetRect(CEikonEnv::Static()->ScreenDevice()->SizeInPixels());

	MakeVisible(EFalse);
	ActivateL();
    }

CBemusedFlipClosedView::~CBemusedFlipClosedView()
    {
    }

TUid CBemusedFlipClosedView::Id() const
    {
    return KFlipClosedViewId;
    }

void CBemusedFlipClosedView::HandleCommandL(TInt aCommand)
    {
    }

void CBemusedFlipClosedView::HandleClientRectChange()
    {
    }

TVwsViewId CBemusedFlipClosedView::ViewId() const
	{
	return TVwsViewId(KUidBemused, Id());
	}

void CBemusedFlipClosedView::ViewActivatedL(const TVwsViewId& aPrevViewId,
		TUid aCustomMessageId, const TDesC8& aCustomMessage)
	{
	TPixelsAndRotation sizeAndRotation;
	CEikonEnv::Static()->ScreenDevice()->GetDefaultScreenSizeAndRotation(sizeAndRotation);
	CEikonEnv::Static()->ScreenDevice()->SetScreenSizeAndRotation(sizeAndRotation);
	CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(EFalse);

	if (iActivateCount == 0)
		MakeVisible(ETrue);
	iActivateCount++;

	iAppUi->SetActiveView(KFlipClosedViewId);
	}

void CBemusedFlipClosedView::ViewDeactivated()
	{
	iActivateCount--;
	if (iActivateCount == 0)
		MakeVisible(EFalse);
	}

void CBemusedFlipClosedView::HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType)
	{
	}

TKeyResponse CBemusedFlipClosedView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	if (aType == EEventKey)
		{
		switch (aKeyEvent.iCode)
			{
		case '2':
			if (iAppUi->CurrentVolume() < (256 - KVolumeStep))
				iAppUi->SetVolume(iAppUi->CurrentVolume() + KVolumeStep);
			else
				iAppUi->SetVolume(255);
			return EKeyWasConsumed;
			break;

		case '0':
			if (iAppUi->CurrentVolume() >= KVolumeStep)
				iAppUi->SetVolume(iAppUi->CurrentVolume() - KVolumeStep);
			else
				iAppUi->SetVolume(0);
			return EKeyWasConsumed;
			break;

		case 63564: // "C"
			iAppUi->HandleCommandL(EEikCmdExit);
			break;

		default:
//			{
//			TBuf<256> formatBuf;
//			formatBuf.Format(_L("code: %d scancode: %d type: %d"), aKeyEvent.iCode,
//				aKeyEvent.iScanCode, aType);
//			CEikonEnv::Static()->InfoWinL(_L("Key event"), formatBuf);
//			}
			break;
			}
		}

	switch (aKeyEvent.iScanCode)
		{
	case EStdKeyDevice1: // jog dial up
		if ((aType == EEventKey) && (iFocusIndex > 0))
			{
			iFocusIndex--;
			iAppUi->DrawControlView(KBemusedDrawButtons);
			}
		break;

	case EStdKeyDevice2: // jog dial down
		if ((aType == EEventKey) && (iFocusIndex < (KNumButtons-1)))
			{
			iFocusIndex++;
			iAppUi->DrawControlView(KBemusedDrawButtons);
			}
		break;

	case EStdKeyDevice8: // jog dial pressed
	case 179: // "OK"
		if (aType == EEventKeyDown)
			{
			iButtonPressed = ETrue;
			}
		else if (aType == EEventKeyUp)
			{
			if (iCheckedButtonPressedAlready)
				{
				iCheckedButtonPressedAlready = EFalse;
				}
			else if (iButtonPressed)
				{
				ExecuteCommand(iFocusIndex);
				}
			iButtonPressed = EFalse;
			}
		iAppUi->DrawControlView(KBemusedDrawButtons);
		break;

	default:
		return EKeyWasNotConsumed;
		}

	return EKeyWasConsumed;
	}

void CBemusedFlipClosedView::ExecuteCommand(TInt aIndex)
	{
	// Execute one of the commands specified by the buttons.

	switch (aIndex)
		{
	case 0:
		if (iAppUi->PlaylistIndex() > 0)
			iAppUi->PlaylistIndex()--;
		iAppUi->BluetoothHandler()->RequestPreviousTrack();
		break;

#ifdef POWERPOINT_ONLY
	case 2:
#else
	case 1:
#endif
		iAppUi->BluetoothHandler()->RequestPlay();
		break;

#ifndef POWERPOINT_ONLY
	case 2:
		iAppUi->BluetoothHandler()->RequestPause();
		break;
#endif

	case 3:
		iAppUi->BluetoothHandler()->RequestStop((TActionOnStopping)
			iAppUi->Settings().iActionOnStopping);
		break;

#ifdef POWERPOINT_ONLY
	case 1:
#else
	case 4:
#endif
		if (iAppUi->PlaylistIndex() < (iAppUi->Playlist().Count()-1))
			iAppUi->PlaylistIndex()++;
		iAppUi->BluetoothHandler()->RequestNextTrack();
		break;

	default:
		break;
		}
	}

TInt CBemusedFlipClosedView::CountComponentControls() const
	{
	return 0;
	}

CCoeControl* CBemusedFlipClosedView::ComponentControl(TInt aIndex) const
	{
	return NULL;
	}

TVwsViewIdAndMessage CBemusedFlipClosedView::ViewScreenDeviceChangedL()
	{
	return TVwsViewIdAndMessage(TVwsViewId(KUidBemused, KMainViewId));
	}

TBool CBemusedFlipClosedView::ViewScreenModeCompatible(TInt aScreenMode)
	{
	return (aScreenMode == EScreenModeFlipClosed);
	}

void CBemusedFlipClosedView::AppInitiatedDraw() const
	{
	Window().Invalidate();
	ActivateGc();
	Window().BeginRedraw();

	DrawRequiredControls(iAppUi->DrawFlags());

	Window().EndRedraw();
	DeactivateGc();
	}

void CBemusedFlipClosedView::DrawRequiredControls(TUint aRequired) const
	{
	CWindowGc& gc = SystemGc();

	if (!iSkin.iValid)
		{
		gc.SetPenStyle(CGraphicsContext::ENullPen);
		gc.SetBrushColor(KRgbGray);
		gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
		gc.DrawRect(Rect());

		iAppUi->SetDrawPending(EFalse);
		iAppUi->SetDrawFlags(KBemusedDrawEverything);

		return;
		}

	if (aRequired & KBemusedDrawBackground)
		{
		TRgb buttonBg;
		iSkin.iBitmap->GetPixel(buttonBg, iSkin.iButtonsRect.iTl - TSize(1, 1));
		gc.SetBrushColor(buttonBg);
		gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
		gc.Clear();
		}

	if (aRequired & KBemusedDrawTrackTime)
		DrawTrackName(Rect());
	if (aRequired & KBemusedDrawTrackName)
		DrawTrackTime(Rect());
	if (aRequired & KBemusedDrawButtons)
		DrawButtons(Rect());

	iAppUi->SetDrawPending(EFalse);
	iAppUi->SetDrawFlags(KBemusedDrawEverything);
	}

// When we get a draw request which needs us to redraw everything, do so.
void CBemusedFlipClosedView::Draw(const TRect& /*aRect*/) const
    {
	DrawRequiredControls(KBemusedDrawEverything);
	}

void CBemusedFlipClosedView::DrawTrackName(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	const CFont* font = CEikonEnv::Static()->TitleFont();
	gc.UseFont(font);

	TRgb trackNameBg;
	iSkin.iBitmap->GetPixel(trackNameBg, iSkin.iTrackNameRect.iTl + TSize(3,3));

	TRect trackNameRect(10, 10, 198, 40);
	gc.SetPenSize(TSize(4,4));
	gc.SetPenColor(iSkin.iTrackNameColour);
	gc.SetBrushColor(trackNameBg);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(trackNameRect);

	// Reduce the size of the rectangle, to draw the text more nicely.
	TRect textRect = trackNameRect;
	textRect.Shrink(2, 0);

	TInt trackNameBaseline =
		textRect.Height() / 2 + font->AscentInPixels() / 2;

	TBuf<256> formatBuf;

	if ((iAppUi->SongState() == ESongPlaying) || (iAppUi->SongState() == ESongPaused))
		{
		if ((!iAppUi->Settings().iUseId3Tags) &&
			(iAppUi->Playlist().Count() > iAppUi->PlaylistIndex()) &&
			(!iAppUi->Shuffle()))
			{
			formatBuf.Copy(*(iAppUi->Playlist()[iAppUi->PlaylistIndex()]));

			// Don't display the file extension for MP3s.
			if ((formatBuf.Length() > 4) && (!formatBuf.Right(4).CompareF(_L(".mp3"))))
				formatBuf.Delete(formatBuf.Length() - 4, 4);
			}
		else
			{
			formatBuf.Copy(iAppUi->SongNameGuess());
			}

		// Check that the track name fits into the space available. If not, we scroll it. There's
		// a small delay at the beginning and end of scrolling.

		TInt trackNameChars = font->TextCount(formatBuf, textRect.Width());

		if (trackNameChars == formatBuf.Length())
			{
			gc.DrawText(formatBuf, trackNameRect, trackNameBaseline, CGraphicsContext::ECenter, 0);
			}
		else
			{
			TInt offset =
				(iAppUi->CurrentTime() % ((formatBuf.Length() - trackNameChars + 10) / 2)) * 2;
			offset -= 4;

			if (offset < 0)
				offset = 0;
			else if (offset > (formatBuf.Length() - trackNameChars))
				offset = formatBuf.Length() - trackNameChars;

			TPtrC drawPtr = formatBuf.Mid(offset, trackNameChars);

			gc.DrawText(drawPtr, trackNameRect, trackNameBaseline, CGraphicsContext::ECenter, 0);
			}
		}
	else
		{
		gc.DrawText(_L("Bemused"), trackNameRect, trackNameBaseline,
			CGraphicsContext::ECenter, 0);
		}
	
	gc.DiscardFont();
	}

void CBemusedFlipClosedView::DrawTrackTime(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	const CFont* font = CEikonEnv::Static()->TitleFont();
	gc.UseFont(font);

	TRgb trackTimeBg;
	iSkin.iBitmap->GetPixel(trackTimeBg, iSkin.iTrackTimeRect.iTl + TSize(3,3));

	TRect trackTimeRect(60, 60, 136, 82);
	gc.SetPenSize(TSize(4,4));
	gc.SetPenColor(iSkin.iTrackTimeColour);
	gc.SetBrushColor(trackTimeBg);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(trackTimeRect);

	TInt trackTimeBaseline =
		trackTimeRect.Height() / 2 + font->AscentInPixels() / 2;

	if ((iAppUi->SongState() == ESongPlaying) || (iAppUi->SongState() == ESongPaused))
		{
		if ((iAppUi->SongLength() > 0) && (iAppUi->CurrentTime() >= 0))
			{
			TBuf<256> formatBuf;

			if (iAppUi->Settings().iTimeDisplay == TBemusedSettings::ECountsUp)
				{
				formatBuf.Format(_L("%02d:%02d"), iAppUi->CurrentTime() / 60,
					iAppUi->CurrentTime() % 60);
				}
			else
				{
				TInt timeLeft = iAppUi->SongLength() - iAppUi->CurrentTime();
				if (timeLeft < 0)
					formatBuf.Copy(_L("--:--"));
				else
					formatBuf.Format(_L("-%02d:%02d"), timeLeft / 60, timeLeft % 60);
				}

			gc.DrawText(formatBuf, trackTimeRect, trackTimeBaseline,
				CGraphicsContext::ECenter, 0);
			}
		else
			{
			gc.DrawText(_L("--:--"), trackTimeRect, trackTimeBaseline,
				CGraphicsContext::ECenter, 0);
			}
		}
	else
		{
		gc.DrawText(_L("--:--"), trackTimeRect, trackTimeBaseline,
			CGraphicsContext::ECenter, 0);
		}

	gc.DiscardFont();
	}

void CBemusedFlipClosedView::DrawButtons(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	TInt buttonWidth = iSkin.iButtonsRect.Width() / KNumButtons;
	TInt buttonHeight = iSkin.iButtonsRect.Height();

	TInt border = (aRect.Width() - iSkin.iButtonsRect.Width()) / 2;

	TPoint topLeft(border, 135 - buttonHeight);

	gc.BitBlt(topLeft, iSkin.iBitmap, iSkin.iButtonsRect);

	if (iFocusIndex > -1)
		{
		if (iButtonPressed)
			{
			gc.BitBlt(topLeft + TPoint(iFocusIndex * buttonWidth, 0),
				iSkin.iBitmap,
				TRect(iFocusIndex * buttonWidth, KFullScreenHeight + buttonHeight,
					(iFocusIndex + 1) * buttonWidth, KFullScreenHeight + (buttonHeight * 2)));
			}
		else
			{
			gc.BitBlt(topLeft + TPoint(iFocusIndex * buttonWidth, 0),
				iSkin.iBitmap,
				TRect(iFocusIndex * buttonWidth, KFullScreenHeight,
					(iFocusIndex + 1) * buttonWidth, KFullScreenHeight + buttonHeight));
			}
		}
	}

// End of File

