/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <msvapi.h> // Defines MessageServer::CurrentDriveL()
#include <eikfutil.h>

#include "BemusedInstallSkinDialog.h"
#include "Bemused.hrh"
#include <bemused.rsg>

CBemusedInstallSkinDialog::CBemusedInstallSkinDialog(CBemusedAppUi* aAppUi,
													 TBemusedSettings& aSettings,
													 TBool aInstall)
	: iSettings(aSettings), iAppUi(aAppUi), iInstall(aInstall)
	{
	}

void CBemusedInstallSkinDialog::PreLayoutDynInitL()
	{
	CEikTextListBox* skinList;

	skinList = (CEikTextListBox*) Control(EBemusedInstallSkinId);

	skinList->SetListBoxObserver(this);

	iListItems = new (ELeave) CDesC16ArrayFlat(2);

	if (iInstall)
		{
		iInboxSkinBitmapFileNames = new (ELeave) CDesC16ArrayFlat(2);
		iInboxSkinDescriptionFileNames = new (ELeave) CDesC16ArrayFlat(2);

		GetInboxSkinListL(iListItems);
		GetCopiedSkinListL(KCDriveSkinTextPath, iListItems);
		GetCopiedSkinListL(KCDriveSkinImagePath, iListItems);
		GetCopiedSkinListL(KDDriveSkinTextPath, iListItems);
		GetCopiedSkinListL(KDDriveSkinImagePath, iListItems);
		}
	else
		{
		GetInstalledSkinListL(iListItems);
		}

	skinList->Model()->SetOwnershipType(ELbmOwnsItemArray);
	skinList->Model()->SetItemTextArray(iListItems);
	}

CBemusedInstallSkinDialog::~CBemusedInstallSkinDialog()
	{
	delete iInboxSkinBitmapFileNames;
	delete iInboxSkinDescriptionFileNames;
	}

TBool CBemusedInstallSkinDialog::OkToExitL(TInt /*aKeycode*/)
	{
	return ETrue;
	}

void CBemusedInstallSkinDialog::GetInstalledSkinListL(CDesCArray* aSkinArray)
	{
	// Gets the list of skins in the Bemused skins directory.

	CDir* entryList;
	TFileName skinPath;
	TInt ret;

	skinPath.Format(_L("%S\\%S"), &iAppUi->SkinPathL(), &KMatchFullScreenSkinBitmaps);

	// Get a list of entries in the skins directory matching the criteria.

	ret = CEikonEnv::Static()->FsSession().GetDir(skinPath, KEntryAttNormal,
		ESortByName, entryList);

	TBuf<256> formatBuf;

	if (ret != KErrNone)
		{
		delete entryList;

		formatBuf.Format(_L("Couldn't get list of skins: %d"), ret);
		CEikonEnv::Static()->InfoWinL(KNullDesC, formatBuf);
		return;
		}

	CleanupStack::PushL(entryList);

	TInt entryCount = entryList->Count();

	// Add all of the entries into the text arrays.

	for (TInt i = 0; i < entryCount; i++)
		{
		formatBuf.Copy((*entryList)[i].iName.Left((*entryList)[i].iName.Length() - 7));
		aSkinArray->AppendL(formatBuf);
		}

	// Make sure that a valid entry is selected.

	if (entryCount <= iSettings.iSkinFileIndex)
		iSettings.iSkinFileIndex = 0;
	if (entryCount <= iSettings.iUninstallSkinIndex)
		iSettings.iUninstallSkinIndex = 0;

	CleanupStack::PopAndDestroy(); // entryList
	}

void CBemusedInstallSkinDialog::GetCopiedSkinListL(const TDesC& aPath, CDesCArray* aSkinArray)
	{
	// Gets the list of skins in the media files directories..

	CDir* entryList;
	TFileName skinPath;
	TInt ret;

	skinPath.Format(_L("%S%S"), &aPath, &KMatchFullScreenSkins);

	// Get a list of entries in the skins directory matching the criteria.

	ret = CEikonEnv::Static()->FsSession().GetDir(skinPath, KEntryAttNormal,
		ESortByName, entryList);

	TBuf<256> formatBuf;

	// The directory doesn't exist. Oops!
	if (ret != KErrNone)
		{
		delete entryList;
		return;
		}

	CleanupStack::PushL(entryList);

	TInt entryCount = entryList->Count();
	TFileName totalPath;

	// Add all of the entries into the text arrays.

	for (TInt i = 0; i < entryCount; i++)
		{
		totalPath.Format(_L("%S%S"), &aPath, &(*entryList)[i].iName);

		// If we have a skin bitmap file, we need to append it to the text arrays for
		// display. If we have a skin description file, we append it to the list of
		// description files.
		if (!(*entryList)[i].iName.Right(3).CompareF(_L("png")))
			{
//			CEikonEnv::Static()->InfoWinL(_L("Found png file"), totalPath);

			iInboxSkinBitmapFileNames->AppendL(totalPath);

			formatBuf.Copy((*entryList)[i].iName.Left((*entryList)[i].iName.Length() - 7));
			aSkinArray->AppendL(formatBuf);
			}
		else if (!(*entryList)[i].iName.Right(3).CompareF(_L("txt")))
			{
//			CEikonEnv::Static()->InfoWinL(_L("Found txt file"), totalPath);
			iInboxSkinDescriptionFileNames->AppendL(totalPath);
			}
		}

	// Make sure that a valid entry is selected.

//	if (entryCount <= iSettings.iSkinFileIndex)
//		iSettings.iSkinFileIndex = 0;
//	if (entryCount <= iSettings.iUninstallSkinIndex)
//		iSettings.iUninstallSkinIndex = 0;

	CleanupStack::PopAndDestroy(); // entryList
	}

void CBemusedInstallSkinDialog::GetInboxSkinListL(CDesCArray* aSkinArray)
	{
	// This function gets a list of skin files in the Inbox.

	// Find the Inbox storage drive
	TInt msgStoreDrv = MessageServer::CurrentDriveL(CEikonEnv::Static()->FsSession());
	TBuf<256> inboxPathAndDrive;
	inboxPathAndDrive.Format(_L("%C:%S"), 'A'+(msgStoreDrv-EDriveA), &KInboxPath);

	// Specify the Inbox directory.
	TBuf<256> totalPath;
	totalPath.Format(_L("%S\\%S"), &inboxPathAndDrive, &KMatchAll);

	// TEntryArrays are very large objects, so we allocate them on the heap.
	TEntryArray* entryArray1 = new (ELeave) TEntryArray;
	CleanupStack::PushL(entryArray1);

	TEntryArray* entryArray2 = new (ELeave) TEntryArray;
	CleanupStack::PushL(entryArray2);

	TEntryArray* entryArray3 = new (ELeave) TEntryArray;
	CleanupStack::PushL(entryArray3);

	RDir dir;

	// Open the Inbox directory.
	User::LeaveIfError(dir.Open(CEikonEnv::Static()->FsSession(), totalPath, KEntryAttDir));
	dir.Read(*entryArray1);
	dir.Close();

	TBuf<256> formatBuf;

	TInt totalFileCount = 0;

	// entryArray now contains the names of all the directories one level down from
	// the Inbox, ie. "0", "1", etc.
	for (TInt i = 0; i < entryArray1->Count(); i++)
		{
		// Specify each directory one level down from the Inbox in turn.
		totalPath.Format(_L("%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
			&KMatchAll);

		User::LeaveIfError(dir.Open(CEikonEnv::Static()->FsSession(), totalPath, KEntryAttDir));
		dir.Read(*entryArray2);
		dir.Close();

		// entryArray2 now contains the names of all the directories a further level
		// down, ie. "001000a0_F" etc.

		for (TInt j = 0; j < entryArray2->Count(); j++)
			{
			totalPath.Format(_L("%S\\%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
				&(*entryArray2)[j].iName, &KMatchFullScreenSkins);

			TInt rc = dir.Open(CEikonEnv::Static()->FsSession(), totalPath, KEntryAttDir);

			// If we can't open this directory, don't worry - just go on to the next one.
			if (rc != KErrNone)
				continue;

			dir.Read(*entryArray3);
			dir.Close();

			// entryArray3 now contains information about all the files in this
			// directory, hooray!

			for (TInt k = 0; k < entryArray3->Count(); k++)
				{
				totalPath.Format(_L("%S\\%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
					&(*entryArray2)[j].iName, &(*entryArray3)[k].iName);

				// If we have a skin bitmap file, we need to append it to the text arrays for
				// display. If we have a skin description file, we append it to the list of
				// description files.
				if (!(*entryArray3)[k].iName.Right(3).CompareF(_L("png")))
					{
//					CEikonEnv::Static()->InfoWinL(_L("Found png file"), totalPath);

					iInboxSkinBitmapFileNames->AppendL(totalPath);

					formatBuf.Copy((*entryArray3)[k].iName.Left((*entryArray3)[k].iName.Length() - 7));
					aSkinArray->AppendL(formatBuf);
					
					totalFileCount++;
					}
				else if (!(*entryArray3)[k].iName.Right(3).CompareF(_L("txt")))
					{
//					CEikonEnv::Static()->InfoWinL(_L("Found txt file"), totalPath);
					iInboxSkinDescriptionFileNames->AppendL(totalPath);
					}
				}		
			}
		}

	CleanupStack::PopAndDestroy(3); // entryArray1, entryArray2, entryArray3
	}

void CBemusedInstallSkinDialog::HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType)
	{
	if (aEventType == EEventItemClicked)
		{
		CEikTextListBox* skinList;
		skinList = (CEikTextListBox*) Control(EBemusedInstallSkinId);

		TInt index = skinList->CurrentItemIndex();

		if (iInstall)
			{
			TBuf<256> skinWithoutExtension;
			TBuf<256> formatBuf;

			TInt lastBackslash =
				(*iInboxSkinBitmapFileNames)[index].LocateReverse('\\');

			skinWithoutExtension.Copy((*iInboxSkinBitmapFileNames)[index].
				Mid(lastBackslash + 1,
					(*iInboxSkinBitmapFileNames)[index].Length() -
					lastBackslash - 8));

			for (TInt j = 0; j < iInboxSkinDescriptionFileNames->Count(); j++)
				{
				lastBackslash =
					(*iInboxSkinDescriptionFileNames)[j].LocateReverse('\\');
				formatBuf.Copy((*iInboxSkinDescriptionFileNames)[j].
					Mid(lastBackslash + 1,
						(*iInboxSkinDescriptionFileNames)[j].Length() -
						lastBackslash - 8));

				
	//		CEikonEnv::Static()->InfoWinL(formatBuf, skinWithoutExtension);

				if (!formatBuf.CompareF(skinWithoutExtension))
					{
					InstallSkin((*iInboxSkinBitmapFileNames)[index],
						(*iInboxSkinDescriptionFileNames)[j]);

					TryExitL(0);
					}
				}
			}
		else
			{
			TBuf<256> resourceBuf;
			iAppUi->CoeEnv()->ReadResource(resourceBuf, R_UNINSTALL_SKIN_QUERY);

			const TDesC& itemText = skinList->Model()->ItemText(index);

			TBuf<256> formatBuf;
			formatBuf.Format(resourceBuf, &itemText);
			
			if (CEikonEnv::Static()->QueryWinL(formatBuf, KNullDesC))
				{
				formatBuf.Format(_L("%S\\%S_fs.png"), &iAppUi->SkinPathL(), &itemText);
				CEikonEnv::Static()->FsSession().Delete(formatBuf);
				formatBuf.Format(_L("%S\\%S_fs.txt"), &iAppUi->SkinPathL(), &itemText);
				CEikonEnv::Static()->FsSession().Delete(formatBuf);

				TryExitL(0);
				}
			}
		}
	}

void CBemusedInstallSkinDialog::InstallSkin(const TDesC& aBitmapFileName,
											const TDesC& aDescriptionFileName)
	{
	// This function copies the skin bitmap and description files from the Inbox into the Bemused
	// skins directory.

	TInt lastBackslash = aDescriptionFileName.LocateReverse('\\');
	TPtrC filePath = aDescriptionFileName.Left(lastBackslash);
	TPtrC nameWithoutExtension = aDescriptionFileName.Mid(lastBackslash + 1,
		aDescriptionFileName.Length() - lastBackslash - 5);

	TBuf<256> sourceBuf;
	TBuf<256> destBuf;
	TBuf<256> resourceBuf;

	sourceBuf.Format(_L("%S\\%S.txt"), &filePath, &nameWithoutExtension);
	destBuf.Format(_L("%S\\%S.txt"), &iAppUi->SkinPathL(), &nameWithoutExtension);

	TInt ret = EikFileUtils::CopyFile(sourceBuf, destBuf);
	if (ret != KErrNone)
		{
		iAppUi->CoeEnv()->ReadResource(resourceBuf, R_COPY_SKIN_DESCRIPTION_ERROR);
		CEikonEnv::Static()->InfoWinL(KNullDesC, resourceBuf);
		return;
		}

	lastBackslash = aBitmapFileName.LocateReverse('\\');
	filePath.Set(aBitmapFileName.Left(lastBackslash));
	nameWithoutExtension.Set(aBitmapFileName.Mid(lastBackslash + 1,
		aBitmapFileName.Length() - lastBackslash - 5));

	sourceBuf.Format(_L("%S\\%S.png"), &filePath, &nameWithoutExtension);
	destBuf.Format(_L("%S\\%S.png"), &iAppUi->SkinPathL(), &nameWithoutExtension);

	ret = EikFileUtils::CopyFile(sourceBuf, destBuf);
	if (ret != KErrNone)
		{
		iAppUi->CoeEnv()->ReadResource(resourceBuf, R_COPY_SKIN_BITMAP_ERROR);
		CEikonEnv::Static()->InfoWinL(KNullDesC, resourceBuf);

		// delete the skin description file
		destBuf.Format(_L("%S\\%S.txt"), &iAppUi->SkinPathL(), &nameWithoutExtension);
		EikFileUtils::DeleteFile(destBuf);
		return;
		}

	iAppUi->CoeEnv()->ReadResource(resourceBuf, R_SKIN_INSTALLED_MSG);
	CEikonEnv::Static()->InfoWinL(KNullDesC, resourceBuf);
	}

TKeyResponse CBemusedInstallSkinDialog::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	if (aKeyEvent.iScanCode == EStdKeyDevice8)
		{
		HandleListBoxEventL((CEikTextListBox*) Control(EBemusedInstallSkinId),
			EEventItemClicked);
		
		return EKeyWasConsumed;
		}

	return CEikDialog::OfferKeyEventL(aKeyEvent, aType);
	}


// End of File  
