/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eikcmbox.h>
#include <eikedwin.h>

#include "BemusedSettingsDialog.h"
#include "Bemused.hrh"
#include "BemusedMainView.h"
#include <bemused.rsg>

CBemusedSettingsDialog::CBemusedSettingsDialog(CBemusedAppUi* aAppUi, TBemusedSettings& aSettings)
	: iSettings(aSettings), iAppUi(aAppUi)
	{
	}

void CBemusedSettingsDialog::PreLayoutDynInitL()
	{
	CEikComboBox* comboBox;

	comboBox = (CEikComboBox*) Control(EBemusedSkinFileNameId);

	CDesCArray* itemArray = new (ELeave) CDesCArrayFlat(2);

	CleanupStack::PushL(itemArray);
	GetSkinListL(itemArray);
	comboBox->SetArray(itemArray); 
	CleanupStack::Pop(); // itemArray

	const TDesC& text1 = comboBox->Array()->MdcaPoint(iSettings.iSkinFileIndex);
	comboBox->SetTextL(&text1);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedActionOnIncomingCallId);
	comboBox->SetArrayL(R_BEMUSED_INCOMING_CALL_SETTING_LIST_ITEM_ARRAY);
	const TDesC& text2 = comboBox->Array()->MdcaPoint(iSettings.iActionOnIncomingCall);
	comboBox->SetTextL(&text2);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedActionOnStoppingId);
	comboBox->SetArrayL(R_BEMUSED_STOPPING_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text3 = comboBox->Array()->MdcaPoint(iSettings.iActionOnStopping);
	comboBox->SetTextL(&text3);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedSongBrowserFontId);
	comboBox->SetArrayL(R_BEMUSED_BROWSER_FONT_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text4 = comboBox->Array()->MdcaPoint(iSettings.iSongBrowserFont);
	comboBox->SetTextL(&text4);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedRetrieveListId);
	comboBox->SetArrayL(R_BEMUSED_RETRIEVE_LIST_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text6 = comboBox->Array()->MdcaPoint(iSettings.iRetrieveList);
	comboBox->SetTextL(&text6);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedDisplayFilesId);
	comboBox->SetArrayL(R_BEMUSED_DISPLAY_FILES_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text7 = comboBox->Array()->MdcaPoint(iSettings.iDisplayFiles);
	comboBox->SetTextL(&text7);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedConnectToPcId);
	comboBox->SetArrayL(R_BEMUSED_CONNECT_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text8 = comboBox->Array()->MdcaPoint(iSettings.iConnectToPc);
	comboBox->SetTextL(&text8);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedTimeDisplayId);
	comboBox->SetArrayL(R_BEMUSED_TIME_DISPLAY_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text9 = comboBox->Array()->MdcaPoint(iSettings.iTimeDisplay);
	comboBox->SetTextL(&text9);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedUseId3TagsId);
	comboBox->SetArrayL(R_BEMUSED_USE_ID3_TAGS_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text10 = comboBox->Array()->MdcaPoint(iSettings.iUseId3Tags);
	comboBox->SetTextL(&text10);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedBluetoothChannelId);
	comboBox->SetArrayL(R_BEMUSED_BLUETOOTH_CHANNEL_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text11 = comboBox->Array()->MdcaPoint(iSettings.iBluetoothChannel);
	comboBox->SetTextL(&text11);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedSelectingItemId);
	comboBox->SetArrayL(R_BEMUSED_SELECTING_ITEM_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text12 = comboBox->Array()->MdcaPoint(iSettings.iSelectingItem);
	comboBox->SetTextL(&text12);
	comboBox->Edwin()->SetReadOnly(ETrue);

	comboBox = (CEikComboBox*) Control(EBemusedConnectUsingId);
	comboBox->SetArrayL(R_BEMUSED_CONNECT_USING_SETTING_LIST_ITEM_ARRAY); 
	const TDesC& text13 = comboBox->Array()->MdcaPoint(iSettings.iConnectUsing);
	comboBox->SetTextL(&text13);
	comboBox->Edwin()->SetReadOnly(ETrue);
	
	comboBox = (CEikComboBox*) Control(EBemusedDownloadDriveId);

	CDesCArray* driveArray = new (ELeave) CDesCArrayFlat(2);

	CleanupStack::PushL(driveArray);
	PopulateDriveLettersL(driveArray);
	comboBox->SetArray(driveArray); 
	CleanupStack::Pop(); // driveArray

	const TDesC& text14 = comboBox->Array()->MdcaPoint(iSettings.iDownloadDrive);
	comboBox->SetTextL(&text14);
	comboBox->Edwin()->SetReadOnly(ETrue);

	CEikEdwin* edwin = (CEikEdwin*) Control(EBemusedServerAddressId);
	edwin->SetTextL(&(iSettings.iServerAddress));
	}

TBool CBemusedSettingsDialog::OkToExitL(TInt /*aKeycode*/)
	{
	TInt oldSkinFileIndex = iSettings.iSkinFileIndex;

	iSettings.iSkinFileIndex =
		GetComboIndex((CEikComboBox*) Control(EBemusedSkinFileNameId));

	if (oldSkinFileIndex != iSettings.iSkinFileIndex)
		iAppUi->ControlView()->SkinHandler()->LoadSkinL();

	iSettings.iActionOnIncomingCall =
		GetComboIndex((CEikComboBox*) Control(EBemusedActionOnIncomingCallId));

	iSettings.iActionOnStopping =
		GetComboIndex((CEikComboBox*) Control(EBemusedActionOnStoppingId));

	iSettings.iSongBrowserFont =
		GetComboIndex((CEikComboBox*) Control(EBemusedSongBrowserFontId));

	iSettings.iRetrieveList =
		GetComboIndex((CEikComboBox*) Control(EBemusedRetrieveListId));

	iSettings.iDisplayFiles =
		GetComboIndex((CEikComboBox*) Control(EBemusedDisplayFilesId));

	iSettings.iConnectToPc =
		GetComboIndex((CEikComboBox*) Control(EBemusedConnectToPcId));

	iSettings.iTimeDisplay =
		GetComboIndex((CEikComboBox*) Control(EBemusedTimeDisplayId));

	iSettings.iUseId3Tags =
		GetComboIndex((CEikComboBox*) Control(EBemusedUseId3TagsId));

	iSettings.iBluetoothChannel =
		GetComboIndex((CEikComboBox*) Control(EBemusedBluetoothChannelId));

	iSettings.iSelectingItem =
		GetComboIndex((CEikComboBox*) Control(EBemusedSelectingItemId));

	iSettings.iDownloadDrive =
		GetComboIndex((CEikComboBox*) Control(EBemusedDownloadDriveId));

	iSettings.iConnectUsing =
		GetComboIndex((CEikComboBox*) Control(EBemusedConnectUsingId));

	((CEikEdwin*) Control(EBemusedServerAddressId))->
		GetText(iSettings.iServerAddress);

	return ETrue;
	}

TInt CBemusedSettingsDialog::GetComboIndex(CEikComboBox* aCombo)
	{
	TBuf<256> text;
	aCombo->GetText(text);

	for (TInt i = 0; i < aCombo->Array()->MdcaCount(); i++)
		{
		if (aCombo->Array()->MdcaPoint(i) == text)
			return i;
		}

	return 0;
	}

void CBemusedSettingsDialog::GetSkinListL(CDesCArray* aSkinArray)
	{
	// Gets the list of skins in the Bemused skins directory.

	CDir* entryList;
	TFileName skinPath;
	TInt ret;

	skinPath.Format(_L("%S\\%S"), &iAppUi->SkinPathL(), &KMatchFullScreenSkinBitmaps);

	// Get a list of entries in the skins directory matching the criteria.

	ret = CEikonEnv::Static()->FsSession().GetDir(skinPath, KEntryAttNormal,
		ESortByName, entryList);

	TBuf<256> formatBuf;

	if (ret != KErrNone)
		{
		delete entryList;

		formatBuf.Format(_L("Couldn't get list of skins: %d"), ret);
		CEikonEnv::Static()->InfoWinL(KNullDesC, formatBuf);
		return;
		}

	CleanupStack::PushL(entryList);

	TInt entryCount = entryList->Count();

	// Add all of the entries into the text arrays.

	for (TInt i = 0; i < entryCount; i++)
		{
		formatBuf.Copy((*entryList)[i].iName.Left((*entryList)[i].iName.Length() - 7));
		aSkinArray->AppendL(formatBuf);
		}

	// Make sure that a valid entry is selected.

	if (entryCount <= iSettings.iSkinFileIndex)
		iSettings.iSkinFileIndex = 0;
	if (entryCount <= iSettings.iUninstallSkinIndex)
		iSettings.iUninstallSkinIndex = 0;

	CleanupStack::PopAndDestroy(); // entryList
	}

void CBemusedSettingsDialog::PopulateDriveLettersL(CDesCArray* aDriveArray)
	{
	TDriveList driveList;
	User::LeaveIfError(CEikonEnv::Static()->FsSession().DriveList(driveList));

	TBuf<1> driveLetter;
	driveLetter.SetMax();
	TInt count = 0;
	for (TInt i = 1; i < 25; i++)
		{
		if (driveList[i])
			{
			driveLetter[0] = 'A' + i;
			aDriveArray->AppendL(driveLetter);
			count++;
			}
		}
	}


// End of File  
